"""
Module de scraping des réseaux sociaux (LinkedIn, Twitter, GitHub, etc.)
"""

import logging
import re
import json
import time
from typing import Dict, List, Any, Optional
from urllib.parse import urlparse, urljoin
import requests
from bs4 import BeautifulSoup

logger = logging.getLogger(__name__)

class SocialScraper:
    """
    Effectue le scraping des réseaux sociaux pour rechercher des informations
    """
    
    def __init__(self, config):
        """
        Initialise le scraper de réseaux sociaux
        
        Args:
            config: Configuration du crawler
        """
        self.config = config
        self.session = requests.Session()
        
        # Configuration de l'user-agent
        if self.config.user_agent_rotation:
            try:
                from fake_useragent import UserAgent
                self.ua = UserAgent()
                self.session.headers.update({"User-Agent": self.ua.random})
            except ImportError:
                logger.warning("Module fake_useragent non disponible. Utilisation de l'user-agent par défaut.")
                self.session.headers.update({
                    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
                })
        else:
            self.session.headers.update({
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
            })
        
        # Configuration du proxy si nécessaire
        if self.config.proxy:
            self.session.proxies = {
                "http": self.config.proxy,
                "https": self.config.proxy
            }
        
        logger.info("Scraper de réseaux sociaux initialisé")
    
    def scrape(self, target: str) -> Dict[str, Any]:
        """
        Effectue le scraping des réseaux sociaux pour la cible
        
        Args:
            target: URL ou nom de domaine cible
            
        Returns:
            Dictionnaire des données recueillies
        """
        logger.info(f"Démarrage du scraping de réseaux sociaux pour {target}")
        
        # Extraire le domaine du site cible
        domain = target
        if target.startswith(("http://", "https://")):
            parsed_url = urlparse(target)
            domain = parsed_url.netloc
        
        # Extraire le nom de l'organisation ou de l'entreprise
        company_name = self._extract_company_name(domain, target)
        
        results = {
            "domain": domain,
            "company": company_name,
            "linkedin": self._scrape_linkedin(company_name),
            "twitter": self._scrape_twitter(company_name, domain),
            "github": self._scrape_github(company_name, domain),
            "facebook": {},
            "instagram": {},
            "profiles": [],
            "mentions": []
        }
        
        return results
    
    def _extract_company_name(self, domain: str, url: str) -> str:
        """
        Tente d'extraire le nom de l'entreprise à partir du domaine ou de la page d'accueil
        """
        # Extraire du domaine (enlever l'extension)
        company = domain.split(".")[0]
        
        # Essayer d'obtenir plus d'informations depuis la page d'accueil
        try:
            response = self.session.get(url, timeout=self.config.timeout)
            if response.status_code == 200:
                soup = BeautifulSoup(response.text, "html.parser")
                
                # Rechercher dans les balises meta, title, etc.
                title = soup.find("title")
                if title and title.text:
                    # Extraire le premier mot significatif (supposé être le nom de l'entreprise)
                    title_text = title.text.split("|")[0].split("-")[0].strip()
                    if len(title_text.split()) <= 3:  # Max 3 mots pour un nom d'entreprise
                        company = title_text
                
                # Rechercher dans les meta tags
                meta_org = soup.find("meta", property="og:site_name")
                if meta_org and meta_org.get("content"):
                    company = meta_org.get("content")
        except Exception as e:
            logger.error(f"Erreur lors de l'extraction du nom de l'entreprise: {str(e)}")
        
        logger.info(f"Nom d'entreprise extrait: {company}")
        return company
    
    def _scrape_linkedin(self, company_name: str) -> Dict[str, Any]:
        """
        Recherche des informations sur LinkedIn
        
        Note: Cette méthode utilise une approche basique sans API pour éviter les limitations.
        Pour un usage professionnel, LinkedIn API serait nécessaire.
        """
        results = {
            "company_profile": "",
            "employees": [],
            "job_titles": set(),
            "locations": set()
        }
        
        # URL de recherche LinkedIn (public)
        search_url = f"https://www.linkedin.com/company/{company_name.lower().replace(' ', '-')}"
        
        try:
            logger.info(f"Recherche de profil LinkedIn pour {company_name}")
            
            # Vérifier si le profil d'entreprise existe
            response = self.session.get(search_url, timeout=self.config.timeout)
            
            if response.status_code == 200:
                results["company_profile"] = search_url
                
                # Extraire des infos basiques (limité sans JavaScript et authentification)
                soup = BeautifulSoup(response.text, "html.parser")
                
                # Extraction d'informations basiques
                description = soup.find("p", {"class": "org-about-us-organization-description__text"})
                if description:
                    results["description"] = description.text.strip()
                
                # Note: Pour une extraction plus complète, il faudrait:
                # 1. Utiliser Selenium/Playwright pour exécuter JavaScript
                # 2. S'authentifier à LinkedIn (contre les TOS de LinkedIn)
                # 3. Utiliser l'API LinkedIn officielle (nécessite une approbation)
                
                logger.info(f"Profil LinkedIn trouvé: {search_url}")
            else:
                logger.info(f"Aucun profil LinkedIn trouvé pour {company_name}")
            
            # Respecter les limites de requêtes
            time.sleep(self.config.request_delay)
            
        except Exception as e:
            logger.error(f"Erreur lors du scraping LinkedIn: {str(e)}")
        
        return results
    
    def _scrape_twitter(self, company_name: str, domain: str) -> Dict[str, Any]:
        """
        Recherche des informations sur Twitter
        """
        results = {
            "profile": "",
            "handle": "",
            "recent_tweets": [],
            "mentions": []
        }
        
        try:
            logger.info(f"Recherche de profil Twitter pour {company_name}")
            
            # Approche 1: Recherche directe par nom d'entreprise
            handles_to_try = [
                company_name.lower().replace(" ", ""),
                company_name.lower().replace(" ", "_"),
                domain.split(".")[0].lower()
            ]
            
            for handle in handles_to_try:
                profile_url = f"https://twitter.com/{handle}"
                response = self.session.get(profile_url, timeout=self.config.timeout)
                
                if response.status_code == 200 and "Account suspended" not in response.text:
                    results["profile"] = profile_url
                    results["handle"] = handle
                    
                    # Extraction basique d'informations (limité sans API)
                    soup = BeautifulSoup(response.text, "html.parser")
                    
                    # Note: Twitter utilise beaucoup de JS, donc l'extraction directe est limitée
                    # Pour une extraction complète, il faudrait:
                    # 1. Utiliser l'API Twitter (nécessite une clé API)
                    # 2. Ou utiliser SNScrape (qui simule un navigateur)
                    
                    logger.info(f"Profil Twitter trouvé: {profile_url}")
                    break
                
                time.sleep(self.config.request_delay)
            
            # Recherche de mentions récentes (nécessiterait l'API Twitter ou SNScrape)
            # Ceci est un placeholder - en production, utiliser l'API Twitter ou SNScrape
            
            if not results["profile"]:
                logger.info(f"Aucun profil Twitter trouvé pour {company_name}")
            
        except Exception as e:
            logger.error(f"Erreur lors du scraping Twitter: {str(e)}")
        
        return results
    
    def _scrape_github(self, company_name: str, domain: str) -> Dict[str, Any]:
        """
        Recherche des informations sur GitHub
        """
        results = {
            "organization": "",
            "repositories": [],
            "members": [],
            "languages": set()
        }
        
        try:
            logger.info(f"Recherche de profil GitHub pour {company_name}")
            
            # Approche 1: Recherche directe par nom d'entreprise
            org_names_to_try = [
                company_name.lower().replace(" ", ""),
                company_name.lower().replace(" ", "-"),
                domain.split(".")[0].lower()
            ]
            
            # Vérifier si nous avons une clé API GitHub
            headers = {}
            if self.config.get("api_keys.github"):
                headers["Authorization"] = f"token {self.config.get('api_keys.github')}"
            
            for org_name in org_names_to_try:
                # Utiliser l'API GitHub (même sans authentification, permet quelques requêtes)
                api_url = f"https://api.github.com/orgs/{org_name}"
                response = self.session.get(api_url, headers=headers, timeout=self.config.timeout)
                
                if response.status_code == 200:
                    org_data = response.json()
                    results["organization"] = org_data.get("html_url", "")
                    results["description"] = org_data.get("description", "")
                    
                    # Récupérer les repositories publics
                    repos_url = f"https://api.github.com/orgs/{org_name}/repos"
                    repos_response = self.session.get(repos_url, headers=headers, timeout=self.config.timeout)
                    
                    if repos_response.status_code == 200:
                        repos = repos_response.json()
                        for repo in repos[:min(10, len(repos))]:  # Limiter à 10 repos
                            results["repositories"].append({
                                "name": repo.get("name", ""),
                                "url": repo.get("html_url", ""),
                                "description": repo.get("description", ""),
                                "stars": repo.get("stargazers_count", 0),
                                "language": repo.get("language", "")
                            })
                            
                            if repo.get("language"):
                                results["languages"].add(repo.get("language"))
                    
                    logger.info(f"Profil GitHub trouvé: {results['organization']}")
                    break
                
                time.sleep(self.config.request_delay)
            
            if not results["organization"]:
                logger.info(f"Aucun profil GitHub trouvé pour {company_name}")
            
        except Exception as e:
            logger.error(f"Erreur lors du scraping GitHub: {str(e)}")
        
        # Convertir les sets en listes pour la sérialisation JSON
        results["languages"] = list(results["languages"])
        
        return results 


"""
Module d'exportation des résultats dans différents formats
"""

import logging
import json
import csv
import os
from typing import Dict, Any, List, Optional
import datetime

logger = logging.getLogger(__name__)

class Exporter:
    """
    Gère l'exportation des résultats dans différents formats
    """
    
    def __init__(self, config):
        """
        Initialise l'exportateur
        
        Args:
            config: Configuration du crawler
        """
        self.config = config
        logger.info("Exportateur initialisé")
    
    def export(self, data: Dict[str, Any], format: str = "json", output: str = "rapport") -> str:
        """
        Exporte les données dans le format spécifié
        
        Args:
            data: Données à exporter
            format: Format d'exportation (json, csv, html, pdf)
            output: Nom de fichier de sortie (sans extension)
            
        Returns:
            Chemin du fichier exporté
        """
        if not data:
            logger.warning("Aucune donnée à exporter")
            return ""
        
        format = format.lower()
        timestamp = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
        filename = f"{output}_{timestamp}"
        
        if format == "json":
            return self.export_json(data, filename)
        elif format == "csv":
            return self.export_csv(data, filename)
        elif format == "html":
            return self.export_html(data, filename)
        elif format == "pdf":
            return self.export_pdf(data, filename)
        else:
            logger.warning(f"Format d'exportation non pris en charge: {format}")
            return self.export_json(data, filename)  # Format par défaut
    
    def export_json(self, data: Dict[str, Any], filename: str) -> str:
        """
        Exporte les données au format JSON
        
        Args:
            data: Données à exporter
            filename: Nom de fichier (sans extension)
            
        Returns:
            Chemin du fichier exporté
        """
        output_file = f"{filename}.json"
        logger.info(f"Exportation des données au format JSON: {output_file}")
        
        try:
            with open(output_file, "w", encoding="utf-8") as f:
                json.dump(data, f, ensure_ascii=False, indent=2, default=str)
            
            logger.info(f"Exportation JSON terminée: {output_file}")
            return os.path.abspath(output_file)
        
        except Exception as e:
            logger.error(f"Erreur lors de l'exportation JSON: {str(e)}")
            return ""
    
    def export_csv(self, data: Dict[str, Any], filename: str) -> str:
        """
        Exporte les données au format CSV
        
        Args:
            data: Données à exporter
            filename: Nom de fichier (sans extension)
            
        Returns:
            Chemin du fichier exporté
        """
        output_file = f"{filename}.csv"
        logger.info(f"Exportation des données au format CSV: {output_file}")
        
        try:
            # Le CSV n'est pas idéal pour les données hiérarchiques
            # Nous allons donc exporter les sections principales dans des fichiers séparés
            
            # Déterminer quelle section exporter
            if "vulnerabilities" in data:
                # Exporter les vulnérabilités
                self._export_vulnerabilities_csv(data, filename)
                return os.path.abspath(f"{filename}_vulnerabilities.csv")
            
            elif "endpoints" in data:
                # Exporter les endpoints API
                self._export_endpoints_csv(data, filename)
                return os.path.abspath(f"{filename}_endpoints.csv")
            
            elif "metadata" in data and "interesting_metadata" in data["metadata"]:
                # Exporter les métadonnées intéressantes
                self._export_metadata_csv(data, filename)
                return os.path.abspath(f"{filename}_metadata.csv")
            
            else:
                # Exportation générique (limité aux valeurs de premier niveau)
                with open(output_file, "w", newline="", encoding="utf-8") as f:
                    writer = csv.writer(f)
                    writer.writerow(["Clé", "Valeur"])
                    
                    for key, value in data.items():
                        if isinstance(value, (str, int, float, bool)):
                            writer.writerow([key, value])
                        elif value is None:
                            writer.writerow([key, ""])
                        else:
                            writer.writerow([key, str(type(value))])
                
                logger.info(f"Exportation CSV générique terminée: {output_file}")
                return os.path.abspath(output_file)
        
        except Exception as e:
            logger.error(f"Erreur lors de l'exportation CSV: {str(e)}")
            return ""
    
    def _export_vulnerabilities_csv(self, data: Dict[str, Any], base_filename: str) -> str:
        """
        Exporte les vulnérabilités au format CSV
        """
        output_file = f"{base_filename}_vulnerabilities.csv"
        
        try:
            with open(output_file, "w", newline="", encoding="utf-8") as f:
                writer = csv.writer(f)
                writer.writerow(["Type", "Sévérité", "URL", "Description"])
                
                for vuln in data.get("potential_vulnerabilities", []):
                    writer.writerow([
                        vuln.get("type", ""),
                        vuln.get("severity", ""),
                        vuln.get("url", ""),
                        vuln.get("description", "")
                    ])
                
                for issue in data.get("header_issues", []):
                    writer.writerow([
                        f"header_{issue.get('header', '')}",
                        issue.get("severity", ""),
                        data.get("target", ""),
                        issue.get("description", "")
                    ])
            
            logger.info(f"Exportation CSV des vulnérabilités terminée: {output_file}")
            return os.path.abspath(output_file)
            
        except Exception as e:
            logger.error(f"Erreur lors de l'exportation CSV des vulnérabilités: {str(e)}")
            return ""
    
    def _export_endpoints_csv(self, data: Dict[str, Any], base_filename: str) -> str:
        """
        Exporte les endpoints API au format CSV
        """
        output_file = f"{base_filename}_endpoints.csv"
        
        try:
            with open(output_file, "w", newline="", encoding="utf-8") as f:
                writer = csv.writer(f)
                writer.writerow(["URL", "Méthode", "Paramètres"])
                
                for endpoint in data.get("endpoints", {}).get("endpoints", []):
                    writer.writerow([
                        endpoint.get("url", ""),
                        endpoint.get("method", "GET"),
                        ", ".join(endpoint.get("parameters", []))
                    ])
            
            logger.info(f"Exportation CSV des endpoints API terminée: {output_file}")
            return os.path.abspath(output_file)
            
        except Exception as e:
            logger.error(f"Erreur lors de l'exportation CSV des endpoints: {str(e)}")
            return ""
    
    def _export_metadata_csv(self, data: Dict[str, Any], base_filename: str) -> str:
        """
        Exporte les métadonnées au format CSV
        """
        output_file = f"{base_filename}_metadata.csv"
        
        try:
            with open(output_file, "w", newline="", encoding="utf-8") as f:
                writer = csv.writer(f)
                writer.writerow(["Fichier", "Type", "Champ", "Valeur"])
                
                for item in data.get("metadata", {}).get("interesting_metadata", []):
                    filename = item.get("filename", "")
                    filetype = item.get("type", "")
                    
                    for field, value in item.get("interesting_fields", {}).items():
                        writer.writerow([filename, filetype, field, value])
            
            logger.info(f"Exportation CSV des métadonnées terminée: {output_file}")
            return os.path.abspath(output_file)
            
        except Exception as e:
            logger.error(f"Erreur lors de l'exportation CSV des métadonnées: {str(e)}")
            return ""
    
    def export_html(self, data: Dict[str, Any], filename: str) -> str:
        """
        Exporte les données au format HTML
        
        Args:
            data: Données à exporter
            filename: Nom de fichier (sans extension)
            
        Returns:
            Chemin du fichier exporté
        """
        output_file = f"{filename}.html"
        logger.info(f"Exportation des données au format HTML: {output_file}")
        
        try:
            with open(output_file, "w", encoding="utf-8") as f:
                # Début du document HTML
                f.write("""
                <!DOCTYPE html>
                <html lang="fr">
                <head>
                    <meta charset="UTF-8">
                    <meta name="viewport" content="width=device-width, initial-scale=1.0">
                    <title>Rapport de Scan - HakBoardCrawler</title>
                    <style>
                        body { font-family: Arial, sans-serif; margin: 20px; color: #333; }
                        h1 { color: #2c3e50; }
                        h2 { color: #3498db; margin-top: 30px; }
                        h3 { color: #2980b9; }
                        .container { max-width: 1200px; margin: 0 auto; }
                        .section { margin-bottom: 30px; padding: 20px; border-radius: 5px; background-color: #f9f9f9; }
                        .info { color: #16a085; }
                        .warning { color: #f39c12; }
                        .danger { color: #c0392b; }
                        table { width: 100%; border-collapse: collapse; margin: 15px 0; }
                        th, td { padding: 10px; text-align: left; border-bottom: 1px solid #ddd; }
                        th { background-color: #f2f2f2; }
                        tr:hover { background-color: #f5f5f5; }
                        .footer { margin-top: 50px; text-align: center; font-size: 0.8em; color: #7f8c8d; }
                    </style>
                </head>
                <body>
                    <div class="container">
                        <h1>Rapport de Scan - HakBoardCrawler</h1>
                """)
                
                # Informations générales
                f.write(f"""
                        <div class="section">
                            <h2>Informations Générales</h2>
                            <table>
                                <tr><th>Cible</th><td>{data.get('target', 'N/A')}</td></tr>
                                <tr><th>Date</th><td>{data.get('timestamp', 'N/A')}</td></tr>
                                <tr><th>Mode</th><td>{data.get('mode', 'N/A')}</td></tr>
                            </table>
                        </div>
                """)
                
                # Vulnérabilités détectées
                if "header_issues" in data or "potential_vulnerabilities" in data:
                    f.write("""
                        <div class="section">
                            <h2>Vulnérabilités Détectées</h2>
                            <table>
                                <tr>
                                    <th>Type</th>
                                    <th>Sévérité</th>
                                    <th>Description</th>
                                </tr>
                    """)
                    
                    for issue in data.get("header_issues", []):
                        severity_class = "info"
                        if issue.get("severity") == "high":
                            severity_class = "danger"
                        elif issue.get("severity") == "medium":
                            severity_class = "warning"
                            
                        f.write(f"""
                            <tr>
                                <td>En-tête {issue.get('header', '')}</td>
                                <td class="{severity_class}">{issue.get('severity', '')}</td>
                                <td>{issue.get('description', '')}</td>
                            </tr>
                        """)
                    
                    for vuln in data.get("potential_vulnerabilities", []):
                        severity_class = "info"
                        if vuln.get("severity") == "high":
                            severity_class = "danger"
                        elif vuln.get("severity") == "medium":
                            severity_class = "warning"
                            
                        f.write(f"""
                            <tr>
                                <td>{vuln.get('type', '')}</td>
                                <td class="{severity_class}">{vuln.get('severity', '')}</td>
                                <td>{vuln.get('description', '')}</td>
                            </tr>
                        """)
                    
                    f.write("""
                            </table>
                        </div>
                    """)
                
                # Fichiers sensibles
                if "exposed_files" in data:
                    f.write("""
                        <div class="section">
                            <h2>Fichiers Sensibles Exposés</h2>
                            <table>
                                <tr>
                                    <th>Chemin</th>
                                    <th>Status</th>
                                    <th>Type</th>
                                </tr>
                    """)
                    
                    for file in data.get("exposed_files", []):
                        f.write(f"""
                            <tr>
                                <td>{file.get('path', '')}</td>
                                <td>{file.get('status_code', '')}</td>
                                <td>{file.get('content_type', '')}</td>
                            </tr>
                        """)
                    
                    f.write("""
                            </table>
                        </div>
                    """)
                
                # Endpoints API
                if "endpoints" in data:
                    f.write("""
                        <div class="section">
                            <h2>Endpoints API Détectés</h2>
                            <table>
                                <tr>
                                    <th>URL</th>
                                    <th>Méthode</th>
                                    <th>Paramètres</th>
                                </tr>
                    """)
                    
                    for endpoint in data.get("endpoints", {}).get("endpoints", []):
                        f.write(f"""
                            <tr>
                                <td>{endpoint.get('url', '')}</td>
                                <td>{endpoint.get('method', 'GET')}</td>
                                <td>{', '.join(endpoint.get('parameters', []))}</td>
                            </tr>
                        """)
                    
                    f.write("""
                            </table>
                        </div>
                    """)
                
                # Métadonnées
                if "metadata" in data and "interesting_metadata" in data["metadata"]:
                    f.write("""
                        <div class="section">
                            <h2>Métadonnées Intéressantes</h2>
                            <table>
                                <tr>
                                    <th>Fichier</th>
                                    <th>Type</th>
                                    <th>Champ</th>
                                    <th>Valeur</th>
                                </tr>
                    """)
                    
                    for item in data.get("metadata", {}).get("interesting_metadata", []):
                        filename = item.get("filename", "")
                        filetype = item.get("type", "")
                        
                        for field, value in item.get("interesting_fields", {}).items():
                            f.write(f"""
                                <tr>
                                    <td>{filename}</td>
                                    <td>{filetype}</td>
                                    <td>{field}</td>
                                    <td>{value}</td>
                                </tr>
                            """)
                    
                    f.write("""
                            </table>
                        </div>
                    """)
                
                # Ajout de la section Site Map et Ressources
                f.write("""
                    <div class="section">
                        <h2>Carte du Site</h2>
                """)
                
                # Ajout des statistiques des ressources
                if "summary" in data and "resources" in data["summary"]:
                    resources = data["summary"]["resources"]
                    f.write(f"""
                        <h3>Statistiques des Ressources</h3>
                        <table>
                            <tr>
                                <th>Type de Ressource</th>
                                <th>Nombre</th>
                            </tr>
                            <tr>
                                <td>Pages/URLs Total</td>
                                <td>{data["summary"].get("total_urls", 0)}</td>
                            </tr>
                            <tr>
                                <td>Images</td>
                                <td>{resources.get("images", 0)}</td>
                            </tr>
                            <tr>
                                <td>Scripts</td>
                                <td>{resources.get("scripts", 0)}</td>
                            </tr>
                            <tr>
                                <td>Feuilles de style</td>
                                <td>{resources.get("stylesheets", 0)}</td>
                            </tr>
                            <tr>
                                <td>Polices</td>
                                <td>{resources.get("fonts", 0)}</td>
                            </tr>
                            <tr>
                                <td>Média</td>
                                <td>{resources.get("media", 0)}</td>
                            </tr>
                        </table>
                    """)
                
                # Ajout de la structure du site (liste des pages)
                if "site_map" in data and "nodes" in data["site_map"]:
                    f.write("""
                        <h3>Pages découvertes</h3>
                        <table>
                            <tr>
                                <th>URL</th>
                                <th>Titre</th>
                                <th>Statut</th>
                                <th>Profondeur</th>
                            </tr>
                    """)
                    
                    for node in data["site_map"]["nodes"]:
                        f.write(f"""
                            <tr>
                                <td>{node.get("url", "")}</td>
                                <td>{node.get("title", "")}</td>
                                <td>{node.get("status", "")}</td>
                                <td>{node.get("depth", "")}</td>
                            </tr>
                        """)
                    
                    f.write("""
                        </table>
                    """)
                    
                # Ajout des informations sur les liens externes
                if "site_map" in data and "nodes" in data["site_map"]:
                    external_links = []
                    for node in data["site_map"]["nodes"]:
                        for link in node.get("links", []):
                            if link.get("is_external", False):
                                external_links.append(link)
                    
                    if external_links:
                        f.write("""
                            <h3>Liens Externes</h3>
                            <table>
                                <tr>
                                    <th>URL</th>
                                    <th>Texte</th>
                                </tr>
                        """)
                        
                        for link in external_links:
                            f.write(f"""
                                <tr>
                                    <td>{link.get("url", "")}</td>
                                    <td>{link.get("text", "")}</td>
                                </tr>
                            """)
                        
                        f.write("""
                            </table>
                        """)
                
                f.write("""
                    </div>
                """)
                
                # Fin du document HTML
                f.write("""
                        <div class="footer">
                            <p>Généré par HakBoardCrawler</p>
                        </div>
                    </div>
                </body>
                </html>
                """)
            
            logger.info(f"Exportation HTML terminée: {output_file}")
            return os.path.abspath(output_file)
        
        except Exception as e:
            logger.error(f"Erreur lors de l'exportation HTML: {str(e)}")
            return ""
    
    def export_pdf(self, data: Dict[str, Any], filename: str) -> str:
        """
        Exporte les données au format PDF
        
        Args:
            data: Données à exporter
            filename: Nom de fichier (sans extension)
            
        Returns:
            Chemin du fichier exporté
        """
        output_file = f"{filename}.pdf"
        logger.info(f"Exportation des données au format PDF: {output_file}")
        
        try:
            # Générer d'abord un fichier HTML
            html_file = self.export_html(data, filename)
            
            # Convertir HTML en PDF avec pdfkit
            try:
                import pdfkit
                pdfkit.from_file(html_file, output_file)
                logger.info(f"Exportation PDF terminée: {output_file}")
                return os.path.abspath(output_file)
            except ImportError:
                logger.warning("Module pdfkit non disponible. Exportation PDF impossible.")
                return html_file
            except Exception as e:
                logger.error(f"Erreur lors de la génération du PDF: {str(e)}")
                return html_file
        
        except Exception as e:
            logger.error(f"Erreur lors de l'exportation PDF: {str(e)}")
            return "" 


"""
Module d'extraction de métadonnées à partir de fichiers et d'URL
"""

import logging
import os
import re
import json
import tempfile
from typing import Dict, Any, List, Optional, Tuple
from urllib.parse import urlparse, urljoin
import requests
import concurrent.futures
from bs4 import BeautifulSoup

# Importer les bibliothèques d'extraction de métadonnées si disponibles
try:
    import PyPDF2
    PYPDF2_AVAILABLE = True
except ImportError:
    PYPDF2_AVAILABLE = False

try:
    import exifread
    EXIFREAD_AVAILABLE = True
except ImportError:
    EXIFREAD_AVAILABLE = False

try:
    from PIL import Image
    from PIL.ExifTags import TAGS
    PIL_AVAILABLE = True
except ImportError:
    PIL_AVAILABLE = False

try:
    import docx
    DOCX_AVAILABLE = True
except ImportError:
    DOCX_AVAILABLE = False

from ..config import CrawlerConfig

logger = logging.getLogger(__name__)

class MetadataExtractor:
    """
    Classe pour extraire les métadonnées de divers types de fichiers
    """
    
    # Extensions de fichiers prises en charge
    SUPPORTED_EXTENSIONS = {
        "pdf": [".pdf"],
        "office": [".doc", ".docx", ".xls", ".xlsx", ".ppt", ".pptx"],
        "image": [".jpg", ".jpeg", ".png", ".gif", ".tiff", ".bmp"],
        "audio": [".mp3", ".wav", ".flac", ".ogg", ".m4a"],
        "video": [".mp4", ".avi", ".mov", ".wmv", ".flv", ".mkv"],
        "archive": [".zip", ".rar", ".7z", ".tar", ".gz"],
        "code": [".html", ".xml", ".js", ".css", ".php", ".aspx", ".jsp"]
    }
    
    # Types MIME pris en charge
    SUPPORTED_MIMETYPES = {
        "pdf": ["application/pdf"],
        "office": [
            "application/msword",
            "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
            "application/vnd.ms-excel",
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            "application/vnd.ms-powerpoint",
            "application/vnd.openxmlformats-officedocument.presentationml.presentation"
        ],
        "image": [
            "image/jpeg", "image/png", "image/gif", 
            "image/tiff", "image/bmp", "image/webp"
        ],
        "audio": [
            "audio/mpeg", "audio/wav", "audio/flac", 
            "audio/ogg", "audio/mp4"
        ],
        "video": [
            "video/mp4", "video/avi", "video/quicktime", 
            "video/x-ms-wmv", "video/x-flv", "video/x-matroska"
        ],
        "archive": [
            "application/zip", "application/x-rar-compressed", 
            "application/x-7z-compressed", "application/x-tar", 
            "application/gzip"
        ],
        "code": [
            "text/html", "text/xml", "application/javascript", 
            "text/css", "application/x-php", "application/x-aspx", 
            "application/x-jsp"
        ]
    }
    
    def __init__(self, config: Optional[Any] = None):
        """
        Initialise l'extracteur de métadonnées
        
        Args:
            config: Configuration du crawler
        """
        self.config = config or CrawlerConfig()
        self.session = requests.Session()
        
        # Configuration de la session
        if self.config.user_agent_rotate:
            self.session.headers.update({"User-Agent": self.config.get_user_agent()})
        else:
            self.session.headers.update({"User-Agent": self.config.user_agent or "HakBoardCrawler/1.0"})
        
        # Configuration du proxy si nécessaire
        if self.config.proxy:
            self.session.proxies = {
                "http": self.config.proxy,
                "https": self.config.proxy
            }
        
        # Avertissement si certaines bibliothèques sont manquantes
        if not PYPDF2_AVAILABLE:
            logger.warning("Module PyPDF2 non disponible. L'extraction de métadonnées PDF sera limitée.")
        if not EXIFREAD_AVAILABLE and not PIL_AVAILABLE:
            logger.warning("Modules exifread et PIL non disponibles. L'extraction de métadonnées EXIF sera limitée.")
        if not DOCX_AVAILABLE:
            logger.warning("Module python-docx non disponible. L'extraction de métadonnées DOCX sera limitée.")
        
        logger.info("Extracteur de métadonnées initialisé")
    
    def extract_from_url(self, url: str) -> Dict[str, Any]:
        """
        Extrait les métadonnées à partir d'une URL
        
        Args:
            url: URL du site à analyser
            
        Returns:
            Dictionnaire des métadonnées trouvées
        """
        logger.info(f"Extraction des métadonnées à partir de {url}")
        
        results = {
            "url": url,
            "files": [],
            "interesting_metadata": [],
            "social_metadata": {},
            "html_metadata": {},
            "framework_info": {},
            "static_resources": {
                "images": [],
                "scripts": [],
                "stylesheets": [],
                "fonts": [],
                "media": []
            }
        }
        
        try:
            # Récupérer la page
            response = self.session.get(url, timeout=self.config.timeout)
            
            # Extraire les métadonnées HTML
            html_metadata = self._extract_html_metadata(response.text, url)
            results["html_metadata"] = html_metadata
            
            # Détecter le framework
            framework_info = self._detect_framework(response.text, url)
            results["framework_info"] = framework_info
            
            # Extraire les ressources statiques
            static_resources = self._extract_static_resources(response.text, url)
            results["static_resources"] = static_resources
            
            # Rechercher des liens vers des fichiers
            files = self._find_files_in_html(response.text, url)
            
            # Ajouter les fichiers spécifiques au framework détecté
            if framework_info.get("name"):
                framework_files = self._find_framework_specific_files(url, framework_info["name"])
                files.extend(framework_files)
            
            # Extraire les métadonnées des fichiers trouvés
            extracted_files = []
            interesting_metadata = []
            
            with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
                future_to_file = {
                    executor.submit(self._extract_file_metadata, file_url): file_url
                    for file_url in files[:20]  # Limiter à 20 fichiers
                }
                
                for future in concurrent.futures.as_completed(future_to_file):
                    file_url = future_to_file[future]
                    try:
                        metadata = future.result()
                        if metadata:
                            extracted_files.append(metadata)
                            
                            # Vérifier si les métadonnées sont intéressantes
                            if self._is_interesting_metadata(metadata):
                                interesting_metadata.append({
                                    "filename": metadata.get("filename", ""),
                                    "type": metadata.get("filetype", ""),
                                    "url": metadata.get("url", ""),
                                    "interesting_fields": self._get_interesting_fields(metadata)
                                })
                    except Exception as e:
                        logger.error(f"Erreur lors de l'extraction des métadonnées de {file_url}: {str(e)}")
            
            # Traiter également les images trouvées dans les ressources statiques
            for img in static_resources["images"]:
                try:
                    img_url = img["src"]
                    # Éviter de traiter deux fois la même image
                    if img_url not in [f["url"] for f in extracted_files]:
                        metadata = self._extract_file_metadata(img_url)
                        if metadata:
                            extracted_files.append(metadata)
                            if self._is_interesting_metadata(metadata):
                                interesting_metadata.append({
                                    "filename": metadata.get("filename", ""),
                                    "type": metadata.get("filetype", ""),
                                    "url": metadata.get("url", ""),
                                    "interesting_fields": self._get_interesting_fields(metadata)
                                })
                except Exception as e:
                    logger.error(f"Erreur lors de l'extraction des métadonnées de l'image {img.get('src', '')}: {str(e)}")
            
            results["files"] = extracted_files
            results["interesting_metadata"] = interesting_metadata
            
            logger.info(f"Extraction terminée. {len(extracted_files)} fichiers analysés, {len(interesting_metadata)} avec des métadonnées intéressantes.")
            
        except Exception as e:
            logger.error(f"Erreur lors de l'extraction des métadonnées depuis {url}: {str(e)}")
        
        return results
    
    def extract_from_file(self, file_path: str) -> Dict[str, Any]:
        """
        Extrait les métadonnées à partir d'un fichier local
        
        Args:
            file_path: Chemin vers le fichier
            
        Returns:
            Dictionnaire des métadonnées extraites
        """
        logger.info(f"Extraction des métadonnées à partir du fichier {file_path}")
        
        if not os.path.exists(file_path):
            logger.error(f"Le fichier {file_path} n'existe pas")
            return {}
        
        filename = os.path.basename(file_path)
        ext = os.path.splitext(filename)[1].lower()
        
        # Déterminer le type de fichier
        filetype = None
        for type_name, extensions in self.SUPPORTED_EXTENSIONS.items():
            if ext in extensions:
                filetype = type_name
                break
        
        if not filetype:
            logger.warning(f"Type de fichier non pris en charge: {ext}")
            return {
                "filename": filename,
                "filetype": "unknown",
                "metadata": {}
            }
        
        # Extraire les métadonnées en fonction du type
        metadata = {}
        
        try:
            if filetype == "pdf" and PYPDF2_AVAILABLE:
                metadata = self._extract_pdf_metadata(file_path)
            elif filetype == "image" and (EXIFREAD_AVAILABLE or PIL_AVAILABLE):
                metadata = self._extract_image_metadata(file_path)
            elif filetype == "office" and ext == ".docx" and DOCX_AVAILABLE:
                metadata = self._extract_docx_metadata(file_path)
            
            return {
                "filename": filename,
                "filetype": filetype,
                "metadata": metadata
            }
            
        except Exception as e:
            logger.error(f"Erreur lors de l'extraction des métadonnées de {file_path}: {str(e)}")
            return {
                "filename": filename,
                "filetype": filetype,
                "metadata": {},
                "error": str(e)
            }
    
    def _find_files_in_html(self, html_content: str, base_url: str) -> List[str]:
        """
        Recherche des liens vers des fichiers dans le contenu HTML
        
        Args:
            html_content: Contenu HTML
            base_url: URL de base pour résoudre les liens relatifs
            
        Returns:
            Liste des URLs de fichiers trouvés
        """
        files = []
        
        soup = BeautifulSoup(html_content, "html.parser")
        
        # Rechercher les liens
        for link in soup.find_all("a", href=True):
            href = link["href"]
            
            # Ignorer les ancres et les liens JavaScript
            if href.startswith("#") or href.startswith("javascript:"):
                continue
            
            # Convertir en URL absolue
            if not href.startswith(("http://", "https://")):
                href = urljoin(base_url, href)
            
            # Vérifier si c'est un fichier pris en charge
            parsed_url = urlparse(href)
            path = parsed_url.path.lower()
            
            for type_name, extensions in self.SUPPORTED_EXTENSIONS.items():
                if any(path.endswith(ext) for ext in extensions):
                    files.append(href)
                    break
        
        return files
    
    def _extract_file_metadata(self, file_url: str) -> Optional[Dict[str, Any]]:
        """
        Extrait les métadonnées d'un fichier distant
        
        Args:
            file_url: URL du fichier
            
        Returns:
            Dictionnaire des métadonnées ou None en cas d'erreur
        """
        logger.info(f"Extraction des métadonnées du fichier {file_url}")
        
        try:
            # Télécharger le fichier
            response = self.session.get(
                file_url, 
                timeout=self.config.timeout, 
                stream=True
            )
            
            if response.status_code != 200:
                logger.warning(f"Impossible de télécharger {file_url}: {response.status_code}")
                return None
            
            # Vérifier le type de contenu
            content_type = response.headers.get("Content-Type", "").split(";")[0].lower()
            
            # Déterminer le type de fichier
            filetype = None
            for type_name, mimetypes in self.SUPPORTED_MIMETYPES.items():
                if content_type in mimetypes:
                    filetype = type_name
                    break
            
            if not filetype:
                # Essayer par l'extension
                parsed_url = urlparse(file_url)
                path = parsed_url.path.lower()
                ext = os.path.splitext(path)[1].lower()
                
                for type_name, extensions in self.SUPPORTED_EXTENSIONS.items():
                    if ext in extensions:
                        filetype = type_name
                        break
            
            if not filetype:
                logger.warning(f"Type de fichier non pris en charge: {content_type}")
                return None
            
            # Enregistrer dans un fichier temporaire
            with tempfile.NamedTemporaryFile(delete=False) as tmp:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:
                        tmp.write(chunk)
                tmp_path = tmp.name
            
            # Extraire les métadonnées en fonction du type
            metadata = {}
            filename = os.path.basename(urlparse(file_url).path)
            
            if filetype == "pdf" and PYPDF2_AVAILABLE:
                metadata = self._extract_pdf_metadata(tmp_path)
            elif filetype == "image" and (EXIFREAD_AVAILABLE or PIL_AVAILABLE):
                metadata = self._extract_image_metadata(tmp_path)
            elif filetype == "office" and filename.endswith(".docx") and DOCX_AVAILABLE:
                metadata = self._extract_docx_metadata(tmp_path)
            
            # Supprimer le fichier temporaire
            os.unlink(tmp_path)
            
            return {
                "url": file_url,
                "filename": filename,
                "filetype": filetype,
                "content_type": content_type,
                "metadata": metadata
            }
            
        except Exception as e:
            logger.error(f"Erreur lors de l'extraction des métadonnées de {file_url}: {str(e)}")
            return None
    
    def _extract_html_metadata(self, html_content: str, url: str) -> Dict[str, Any]:
        """
        Extrait les métadonnées du HTML
        
        Args:
            html_content: Contenu HTML
            url: URL de la page
            
        Returns:
            Dictionnaire des métadonnées HTML
        """
        metadata = {
            "title": "",
            "description": "",
            "keywords": [],
            "author": "",
            "generator": "",
            "og": {},
            "twitter": {},
            "meta_tags": []
        }
        
        soup = BeautifulSoup(html_content, "html.parser")
        
        # Titre
        title_tag = soup.find("title")
        if title_tag:
            metadata["title"] = title_tag.text.strip()
        
        # Balises meta
        for meta in soup.find_all("meta"):
            name = meta.get("name", "").lower()
            property = meta.get("property", "").lower()
            content = meta.get("content", "")
            
            metadata["meta_tags"].append({
                "name": name or property,
                "content": content
            })
            
            if name == "description" or property == "og:description":
                metadata["description"] = content
            elif name == "keywords":
                metadata["keywords"] = [k.strip() for k in content.split(",")]
            elif name == "author":
                metadata["author"] = content
            elif name == "generator":
                metadata["generator"] = content
            
            # Open Graph
            if property.startswith("og:"):
                og_key = property[3:]
                metadata["og"][og_key] = content
            
            # Twitter Cards
            if property.startswith("twitter:"):
                twitter_key = property[8:]
                metadata["twitter"][twitter_key] = content
        
        return metadata
    
    def _extract_pdf_metadata(self, file_path: str) -> Dict[str, Any]:
        """
        Extrait les métadonnées d'un fichier PDF
        
        Args:
            file_path: Chemin vers le fichier PDF
            
        Returns:
            Dictionnaire des métadonnées PDF
        """
        if not PYPDF2_AVAILABLE:
            return {}
        
        metadata = {}
        
        try:
            with open(file_path, "rb") as f:
                pdf = PyPDF2.PdfReader(f)
                
                if pdf.metadata:
                    for key, value in pdf.metadata.items():
                        if key.startswith("/"):
                            key = key[1:]
                        metadata[key] = str(value)
                
                metadata["pages"] = len(pdf.pages)
                
                # Extraire le texte de la première page
                if pdf.pages and hasattr(pdf.pages[0], "extract_text"):
                    text = pdf.pages[0].extract_text()
                    if text:
                        metadata["first_page_text"] = text[:1000]  # Limiter à 1000 caractères
        
        except Exception as e:
            logger.error(f"Erreur lors de l'extraction des métadonnées PDF: {str(e)}")
        
        return metadata
    
    def _extract_image_metadata(self, file_path: str) -> Dict[str, Any]:
        """
        Extrait les métadonnées d'une image
        
        Args:
            file_path: Chemin vers le fichier image
            
        Returns:
            Dictionnaire des métadonnées extraites
        """
        metadata = {}
        
        try:
            # Utiliser exifread si disponible
            if EXIFREAD_AVAILABLE:
                with open(file_path, 'rb') as f:
                    tags = exifread.process_file(f, details=True)
                    
                    # Convertir les tags en dictionnaire
                    for tag, value in tags.items():
                        # Ignorer les tags de miniature qui sont volumineux
                        if "thumbnail" not in tag.lower():
                            metadata[tag] = str(value)
            
            # Utiliser PIL comme alternative ou complément
            if PIL_AVAILABLE:
                try:
                    with Image.open(file_path) as img:
                        # Informations de base
                        metadata["ImageWidth"] = img.width
                        metadata["ImageHeight"] = img.height
                        metadata["ImageFormat"] = img.format
                        metadata["ImageMode"] = img.mode
                        
                        # Extraire les données EXIF
                        if hasattr(img, '_getexif') and img._getexif():
                            exif_data = img._getexif()
                            if exif_data:
                                for tag_id, value in exif_data.items():
                                    tag = TAGS.get(tag_id, tag_id)
                                    # Éviter les données binaires volumineuses
                                    if isinstance(value, bytes):
                                        metadata[tag] = f"[Binary data, {len(value)} bytes]"
                                    else:
                                        metadata[tag] = str(value)
                                        
                        # Recherche de métadonnées géographiques
                        geo_tags = ['GPSLatitude', 'GPSLatitudeRef', 'GPSLongitude', 'GPSLongitudeRef',
                                    'GPSAltitude', 'GPSAltitudeRef', 'GPSDateStamp', 'GPSTimeStamp']
                        
                        has_geo = False
                        for tag in geo_tags:
                            if tag in metadata:
                                has_geo = True
                        
                        metadata["HasGeoData"] = has_geo
                except Exception as e:
                    logger.debug(f"Erreur PIL lors de l'extraction des métadonnées image: {str(e)}")
        
        except Exception as e:
            logger.error(f"Erreur lors de l'extraction des métadonnées image: {str(e)}")
        
        return metadata
    
    def _extract_docx_metadata(self, file_path: str) -> Dict[str, Any]:
        """
        Extrait les métadonnées d'un fichier DOCX
        
        Args:
            file_path: Chemin vers le fichier DOCX
            
        Returns:
            Dictionnaire des métadonnées DOCX
        """
        if not DOCX_AVAILABLE:
            return {}
        
        metadata = {}
        
        try:
            doc = docx.Document(file_path)
            
            # Propriétés du document
            core_properties = doc.core_properties
            
            if core_properties:
                metadata["author"] = core_properties.author
                metadata["created"] = str(core_properties.created) if core_properties.created else None
                metadata["last_modified_by"] = core_properties.last_modified_by
                metadata["modified"] = str(core_properties.modified) if core_properties.modified else None
                metadata["title"] = core_properties.title
                metadata["subject"] = core_properties.subject
                metadata["keywords"] = core_properties.keywords
                metadata["comments"] = core_properties.comments
                metadata["category"] = core_properties.category
                metadata["version"] = core_properties.revision
            
            # Statistiques du document
            metadata["paragraphs"] = len(doc.paragraphs)
            metadata["sections"] = len(doc.sections)
            
        except Exception as e:
            logger.error(f"Erreur lors de l'extraction des métadonnées DOCX: {str(e)}")
        
        return metadata
    
    def _is_interesting_metadata(self, file_metadata: Dict[str, Any]) -> bool:
        """
        Détermine si les métadonnées contiennent des informations intéressantes
        
        Args:
            file_metadata: Métadonnées du fichier
            
        Returns:
            True si les métadonnées sont intéressantes, False sinon
        """
        metadata = file_metadata.get("metadata", {})
        
        # Mots-clés intéressants dans les champs
        interesting_fields = [
            "author", "creator", "producer", "last_modified_by",
            "gps", "location", "GPS", "make", "model", "software",
            "email", "company", "organization", "username"
        ]
        
        # Vérifier si des champs intéressants sont présents
        for field in interesting_fields:
            for key in metadata.keys():
                if field.lower() in key.lower():
                    return True
        
        # Vérifier les valeurs
        email_pattern = r'[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}'
        for value in metadata.values():
            if isinstance(value, str) and re.search(email_pattern, value):
                return True
        
        return False
    
    def _get_interesting_fields(self, file_metadata: Dict[str, Any]) -> Dict[str, Any]:
        """
        Extrait les champs intéressants des métadonnées
        
        Args:
            file_metadata: Métadonnées du fichier
            
        Returns:
            Dictionnaire des champs intéressants
        """
        metadata = file_metadata.get("metadata", {})
        interesting = {}
        
        # Mots-clés intéressants dans les champs
        interesting_fields = [
            "author", "creator", "producer", "last_modified_by",
            "gps", "location", "GPS", "make", "model", "software",
            "email", "company", "organization", "username", "title",
            "subject", "keywords", "comments", "category"
        ]
        
        # Extraire les champs intéressants
        for key, value in metadata.items():
            for field in interesting_fields:
                if field.lower() in key.lower():
                    interesting[key] = value
                    break
        
        # Rechercher des emails dans les valeurs
        email_pattern = r'[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}'
        for key, value in metadata.items():
            if isinstance(value, str) and re.search(email_pattern, value) and key not in interesting:
                interesting[key] = value
        
        return interesting
    
    def _extract_static_resources(self, html_content: str, base_url: str) -> Dict[str, List[Dict[str, str]]]:
        """
        Extrait toutes les ressources statiques d'une page HTML
        
        Args:
            html_content: Contenu HTML
            base_url: URL de base pour résoudre les liens relatifs
            
        Returns:
            Dictionnaire des ressources statiques
        """
        resources = {
            "images": [],
            "scripts": [],
            "stylesheets": [],
            "fonts": [],
            "media": []
        }
        
        soup = BeautifulSoup(html_content, "html.parser")
        
        # Extraire les images
        for img in soup.find_all("img", src=True):
            src = img["src"]
            if not src.startswith(("http://", "https://")):
                src = urljoin(base_url, src)
            
            resources["images"].append({
                "src": src,
                "alt": img.get("alt", ""),
                "width": img.get("width", ""),
                "height": img.get("height", "")
            })
        
        # Extraire les scripts
        for script in soup.find_all("script", src=True):
            src = script["src"]
            if not src.startswith(("http://", "https://")):
                src = urljoin(base_url, src)
            
            resources["scripts"].append({
                "src": src
            })
        
        # Extraire les feuilles de style
        for link in soup.find_all("link", rel="stylesheet", href=True):
            href = link["href"]
            if not href.startswith(("http://", "https://")):
                href = urljoin(base_url, href)
            
            resources["stylesheets"].append({
                "href": href
            })
        
        # Extraire les polices
        for link in soup.find_all("link", rel=lambda r: r and "font" in r, href=True):
            href = link["href"]
            if not href.startswith(("http://", "https://")):
                href = urljoin(base_url, href)
            
            resources["fonts"].append({
                "href": href
            })
        
        # Extraire les médias (audio, video)
        for media in soup.find_all(["audio", "video"]):
            src = media.get("src", "")
            if src:
                if not src.startswith(("http://", "https://")):
                    src = urljoin(base_url, src)
                
                resources["media"].append({
                    "src": src,
                    "type": media.name
                })
            
            # Vérifier aussi les balises source dans audio/video
            for source in media.find_all("source", src=True):
                src = source["src"]
                if not src.startswith(("http://", "https://")):
                    src = urljoin(base_url, src)
                
                resources["media"].append({
                    "src": src,
                    "type": source.get("type", "")
                })
        
        # Rechercher les backgrounds dans les styles
        for style in soup.find_all("style"):
            if style.string:
                # Rechercher les URLs dans les styles
                urls = re.findall(r'url\([\'"]?([^\'"]*)[\'"]?\)', style.string)
                for url in urls:
                    if not url.startswith(("http://", "https://")):
                        url = urljoin(base_url, url)
                    
                    # Déterminer le type d'après l'extension
                    ext = os.path.splitext(url)[1].lower()
                    if ext in [".jpg", ".jpeg", ".png", ".gif", ".webp", ".svg"]:
                        resources["images"].append({"src": url})
                    elif ext in [".ttf", ".woff", ".woff2", ".eot"]:
                        resources["fonts"].append({"href": url})
        
        # Rechercher les images de background dans les attributs style
        for elem in soup.find_all(style=True):
            style = elem["style"]
            urls = re.findall(r'url\([\'"]?([^\'"]*)[\'"]?\)', style)
            for url in urls:
                if not url.startswith(("http://", "https://")):
                    url = urljoin(base_url, url)
                
                # Déterminer le type d'après l'extension
                ext = os.path.splitext(url)[1].lower()
                if ext in [".jpg", ".jpeg", ".png", ".gif", ".webp", ".svg"]:
                    resources["images"].append({"src": url})
        
        return resources
    
    def _detect_framework(self, html_content: str, url: str) -> Dict[str, Any]:
        """
        Détecte le framework utilisé par le site
        
        Args:
            html_content: Contenu HTML
            url: URL de base
            
        Returns:
            Informations sur le framework détecté
        """
        framework_info = {
            "name": "",
            "version": "",
            "indicators": []
        }
        
        soup = BeautifulSoup(html_content, "html.parser")
        
        # Détection de Next.js
        next_indicators = [
            soup.find("div", id="__next"),
            soup.find("script", src=lambda s: s and "_next/static" in s),
            soup.find("link", href=lambda h: h and "_next/static" in h)
        ]
        if any(next_indicators):
            framework_info["name"] = "next.js"
            framework_info["indicators"] = [str(i) for i in next_indicators if i]
            
            # Vérifier la version (peut être dans un fichier manifest)
            try:
                manifest_url = urljoin(url, "/_next/static/chunks/webpack.js")
                response = self.session.get(manifest_url, timeout=self.config.timeout)
                if response.status_code == 200:
                    # Essayer de trouver la version
                    version_match = re.search(r'next[\'"]?\s*:\s*[\'"]?([0-9\.]+)', response.text)
                    if version_match:
                        framework_info["version"] = version_match.group(1)
            except Exception as e:
                logger.debug(f"Erreur lors de la détection de la version Next.js: {str(e)}")
        
        # Détection de WordPress
        wp_indicators = [
            soup.find("link", attrs={"rel": "https://api.w.org/"}),
            soup.find("meta", attrs={"name": "generator", "content": lambda c: c and "WordPress" in c}),
            soup.find("script", src=lambda s: s and "wp-includes" in s)
        ]
        if any(wp_indicators):
            framework_info["name"] = "wordpress"
            framework_info["indicators"] = [str(i) for i in wp_indicators if i]
            
            # Vérifier la version dans la balise meta generator
            meta_generator = soup.find("meta", attrs={"name": "generator"})
            if meta_generator and meta_generator.get("content"):
                version_match = re.search(r'WordPress\s+([0-9\.]+)', meta_generator["content"])
                if version_match:
                    framework_info["version"] = version_match.group(1)
        
        # Détection de Django
        django_indicators = [
            soup.find("meta", attrs={"name": "generator", "content": lambda c: c and "Django" in c}),
            soup.find("script", src=lambda s: s and "django" in s)
        ]
        if any(django_indicators):
            framework_info["name"] = "django"
            framework_info["indicators"] = [str(i) for i in django_indicators if i]
        
        # Détection de Laravel
        laravel_indicators = [
            soup.find("meta", attrs={"name": "csrf-token"}),
            soup.find("script", string=lambda s: s and "Laravel" in s),
            soup.find("script", src=lambda s: s and "vendor/laravel" in s)
        ]
        if any(laravel_indicators):
            framework_info["name"] = "laravel"
            framework_info["indicators"] = [str(i) for i in laravel_indicators if i]
        
        # Détection de Vue.js / Nuxt.js
        vue_indicators = [
            soup.find("div", id="app"),
            soup.find("div", id="__nuxt"),
            soup.find("script", src=lambda s: s and "vue.js" in s)
        ]
        if any(vue_indicators):
            framework_info["name"] = "vue.js"
            # Si c'est Nuxt.js spécifiquement
            if soup.find("div", id="__nuxt"):
                framework_info["name"] = "nuxt.js"
            framework_info["indicators"] = [str(i) for i in vue_indicators if i]
        
        # Détection de React
        react_indicators = [
            soup.find("div", id="root"),
            soup.find("script", string=lambda s: s and "ReactDOM" in s),
            soup.find("script", string=lambda s: s and "React.createElement" in s)
        ]
        if any(react_indicators) and not framework_info["name"]:  # Ne pas remplacer Next.js
            framework_info["name"] = "react"
            framework_info["indicators"] = [str(i) for i in react_indicators if i]
        
        return framework_info
    
    def _find_framework_specific_files(self, url: str, framework: str) -> List[str]:
        """
        Recherche des fichiers spécifiques au framework détecté
        
        Args:
            url: URL de base
            framework: Nom du framework détecté
            
        Returns:
            Liste d'URLs de fichiers à vérifier
        """
        files = []
        parsed_url = urlparse(url)
        base_url = f"{parsed_url.scheme}://{parsed_url.netloc}"
        
        if framework == "next.js":
            # Chemins spécifiques à Next.js
            next_paths = [
                "/_next/static/chunks/framework.js",
                "/_next/static/chunks/webpack.js",
                "/_next/static/chunks/main.js",
                "/_next/static/css/",
                "/_next/static/media/",
                "/_next/data/"
            ]
            
            for path in next_paths:
                framework_url = urljoin(base_url, path)
                try:
                    # Vérifier si le chemin existe (pour _next/static/css et _next/static/media, essayer de lister le répertoire)
                    if path.endswith("/"):
                        response = self.session.get(framework_url, timeout=self.config.timeout)
                        if response.status_code == 200:
                            # Essayer de trouver des liens dans la réponse
                            soup = BeautifulSoup(response.text, "html.parser")
                            for a in soup.find_all("a", href=True):
                                href = a["href"]
                                if not href.startswith(("http://", "https://")):
                                    href = urljoin(framework_url, href)
                                files.append(href)
                    else:
                        files.append(framework_url)
                except Exception as e:
                    logger.debug(f"Erreur lors de l'exploration du chemin Next.js {path}: {str(e)}")
        
        elif framework == "wordpress":
            # Chemins spécifiques à WordPress
            wp_paths = [
                "/wp-content/uploads/",
                "/wp-includes/js/",
                "/wp-includes/css/",
                "/wp-content/themes/",
                "/wp-content/plugins/"
            ]
            
            for path in wp_paths:
                framework_url = urljoin(base_url, path)
                try:
                    response = self.session.get(framework_url, timeout=self.config.timeout)
                    if response.status_code == 200:
                        # Essayer de trouver des liens dans la réponse
                        soup = BeautifulSoup(response.text, "html.parser")
                        for a in soup.find_all("a", href=True):
                            href = a["href"]
                            if not href.startswith(("http://", "https://")):
                                href = urljoin(framework_url, href)
                            # Filtrer par extension (images, css, js)
                            if href.endswith((".jpg", ".jpeg", ".png", ".gif", ".css", ".js")):
                                files.append(href)
                except Exception as e:
                    logger.debug(f"Erreur lors de l'exploration du chemin WordPress {path}: {str(e)}")
        
        elif framework in ["vue.js", "nuxt.js"]:
            # Chemins spécifiques à Vue.js/Nuxt.js
            vue_paths = [
                "/_nuxt/",
                "/assets/",
                "/static/"
            ]
            
            for path in vue_paths:
                framework_url = urljoin(base_url, path)
                try:
                    response = self.session.get(framework_url, timeout=self.config.timeout)
                    if response.status_code == 200:
                        soup = BeautifulSoup(response.text, "html.parser")
                        for a in soup.find_all("a", href=True):
                            href = a["href"]
                            if not href.startswith(("http://", "https://")):
                                href = urljoin(framework_url, href)
                            files.append(href)
                except Exception as e:
                    logger.debug(f"Erreur lors de l'exploration du chemin Vue/Nuxt {path}: {str(e)}")
        
        return files 


"""
Module de configuration pour HakBoardCrawler
"""

import json
import logging
import os
from typing import Dict, Any, Optional, List

logger = logging.getLogger(__name__)

class CrawlerConfig:
    """
    Gère la configuration du crawler
    """
    
    def __init__(self, config_file: Optional[str] = None):
        """
        Initialise la configuration
        
        Args:
            config_file: Chemin vers un fichier de configuration JSON (optionnel)
        """
        # Configuration par défaut
        self.scan_depth = 3             # Profondeur maximale d'exploration
        self.threads = 5                # Nombre de threads pour les requêtes parallèles
        self.timeout = 10               # Timeout des requêtes (secondes)
        self.delay = 1.0                # Délai entre les requêtes (secondes)
        self.user_agent_rotate = True   # Rotation des user-agents
        self.user_agent = None          # User-agent spécifique (si rotation désactivée)
        self.respect_robots = True      # Respecter robots.txt
        self.proxy = None               # Proxy à utiliser
        self.cookies = None             # Cookies à inclure dans les requêtes
        self.headers = {}               # En-têtes HTTP additionnelles
        self.verify_ssl = True          # Vérifier les certificats SSL
        self.follow_redirects = True    # Suivre les redirections
        self.max_file_size = 10485760   # Taille maximale des fichiers à télécharger (10 MB)
        self.output_dir = "reports"     # Répertoire de sortie
        self.log_file = None            # Fichier de log
        self.exclude_patterns = []      # Patterns d'URL à exclure
        self.include_patterns = []      # Patterns d'URL à inclure
        self.sensitive_extensions = [   # Extensions de fichiers sensibles à vérifier
            ".env", ".git", ".svn", ".htaccess", ".htpasswd", 
            ".bak", ".backup", ".swp", ".old", ".orig", ".tmp",
            ".sql", ".db", ".sqlite", ".sqlite3", ".mdb", ".psql",
            ".log", ".conf", ".config", ".xml", ".json", ".yml", ".yaml",
            ".ini", ".csv", ".bat", ".sh", "wp-config.php", "config.php"
        ]
        
        # Options d'activation des modules
        self.enable_metadata = True      # Activer l'extraction de métadonnées
        self.enable_sitemap = True       # Activer la génération de la carte du site
        self.enable_api_detection = True # Activer la détection des API
        self.enable_stealth = True       # Activer le mode furtif (rotation des user-agents, etc.)
        
        # Options spécifiques pour la génération de la carte du site
        self.sitemap_options = {
            "extract_scripts": True,     # Extraire les scripts JS
            "extract_styles": True,      # Extraire les feuilles de style
            "extract_images": True,      # Extraire les images
            "extract_links": True,       # Extraire les liens
            "extract_forms": True,       # Extraire les formulaires
            "extract_frames": True,      # Extraire les iframes
            "extract_meta": True         # Extraire les métadonnées
        }
        
        # Options spécifiques pour l'extraction des métadonnées
        self.metadata_options = {
            "extract_images": True,      # Extraire les métadonnées des images
            "extract_pdfs": True,        # Extraire les métadonnées des PDFs
            "extract_docs": True,        # Extraire les métadonnées des documents
            "extract_archives": False,   # Extraire les métadonnées des archives
            "download_files": True,      # Télécharger les fichiers pour extraction
            "max_files": 50              # Nombre maximum de fichiers à analyser
        }
        
        # Options spécifiques pour le scanner de vulnérabilités
        self.scanner_options = {
            "check_headers": True,       # Vérifier les en-têtes de sécurité
            "check_cors": True,          # Vérifier la configuration CORS
            "check_csp": True,           # Vérifier la politique de sécurité du contenu
            "check_exposed_files": True, # Vérifier les fichiers exposés
            "check_sensitive_data": True # Vérifier les données sensibles (emails, tokens, etc.)
        }
        
        # Frameworks à détecter
        self.frameworks = [
            "wordpress", "drupal", "joomla", "magento", "nextjs", "nuxtjs", 
            "react", "angular", "vue", "laravel", "django", "flask", "symfony", 
            "spring", "express", "rails", "aspnet"
        ]
        
        # Chemins des API à vérifier par framework
        self.framework_apis = {
            "wordpress": ["/wp-json/", "/wp-admin/admin-ajax.php"],
            "drupal": ["/jsonapi/", "/user/login", "/node/"],
            "nextjs": ["/_next/data/"],
            "django": ["/admin/", "/api/"],
            "laravel": ["/api/"],
            "springboot": ["/actuator/"],
            "express": ["/api/", "/graphql"],
            "aspnet": ["/api/"]
        }
        
        # Charger la configuration depuis un fichier
        if config_file and os.path.isfile(config_file):
            self.load_config(config_file)
        
    def load_config(self, config_file: str) -> None:
        """
        Charge la configuration depuis un fichier JSON
        
        Args:
            config_file: Chemin vers le fichier de configuration
        """
        try:
            with open(config_file, 'r') as f:
                config = json.load(f)
                
            # Mettre à jour les paramètres
            for key, value in config.items():
                if hasattr(self, key):
                    setattr(self, key, value)
                    
            logger.info(f"Configuration chargée depuis {config_file}")
        except Exception as e:
            logger.error(f"Erreur lors du chargement de la configuration: {str(e)}")
    
    def save_config(self, config_file: str) -> None:
        """
        Sauvegarde la configuration dans un fichier JSON
        
        Args:
            config_file: Chemin vers le fichier de sortie
        """
        try:
            # Créer le dictionnaire de configuration
            config = {}
            for key in dir(self):
                if not key.startswith('__') and not callable(getattr(self, key)):
                    config[key] = getattr(self, key)
            
            # Supprimer les méthodes
            for key in list(config.keys()):
                if callable(config[key]):
                    del config[key]
            
            # Écrire dans le fichier
            with open(config_file, 'w') as f:
                json.dump(config, f, indent=4)
                
            logger.info(f"Configuration sauvegardée dans {config_file}")
        except Exception as e:
            logger.error(f"Erreur lors de la sauvegarde de la configuration: {str(e)}")
    
    def update(self, config_dict: Dict[str, Any]) -> None:
        """
        Met à jour la configuration avec les valeurs fournies
        
        Args:
            config_dict: Dictionnaire des paramètres à mettre à jour
        """
        for key, value in config_dict.items():
            if hasattr(self, key):
                setattr(self, key, value)
                logger.debug(f"Paramètre {key} mis à jour à {value}")
            else:
                logger.warning(f"Paramètre inconnu: {key}")
    
    def get_user_agent(self) -> str:
        """
        Retourne un user-agent aléatoire ou celui défini par l'utilisateur
        
        Returns:
            User-agent à utiliser pour les requêtes
        """
        if not self.user_agent_rotate or self.user_agent:
            return self.user_agent or "HakBoardCrawler/1.0"
        
        # Liste de User-Agents populaires
        user_agents = [
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1.1 Safari/605.1.15",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (iPhone; CPU iPhone OS 14_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.0 Mobile/15E148 Safari/604.1",
            "Mozilla/5.0 (iPad; CPU OS 14_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.0 Mobile/15E148 Safari/604.1",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36 Edg/91.0.864.59"
        ]
        
        import random
        return random.choice(user_agents)
    
    def get_proxy_dict(self) -> Optional[Dict[str, str]]:
        """
        Retourne la configuration du proxy au format attendu par requests
        
        Returns:
            Dictionnaire de configuration du proxy ou None
        """
        if not self.proxy:
            return None
            
        return {
            "http": self.proxy,
            "https": self.proxy
        }
    
    def get_headers(self) -> Dict[str, str]:
        """
        Retourne les en-têtes HTTP pour les requêtes
        
        Returns:
            Dictionnaire des en-têtes HTTP
        """
        headers = {
            "User-Agent": self.get_user_agent(),
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
            "Accept-Language": "en-US,en;q=0.5",
            "Accept-Encoding": "gzip, deflate, br",
            "Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "DNT": "1"
        }
        
        # Ajouter les en-têtes personnalisées
        headers.update(self.headers or {})
        
        return headers
    
    def is_url_allowed(self, url: str) -> bool:
        """
        Vérifie si une URL est autorisée selon les patterns d'inclusion/exclusion
        
        Args:
            url: URL à vérifier
            
        Returns:
            True si l'URL est autorisée, False sinon
        """
        import re
        
        # Vérifier les patterns d'exclusion
        for pattern in self.exclude_patterns:
            if re.search(pattern, url):
                return False
        
        # Vérifier les patterns d'inclusion si définis
        if self.include_patterns:
            for pattern in self.include_patterns:
                if re.search(pattern, url):
                    return True
            return False
        
        # Si aucun pattern d'inclusion n'est défini, autoriser par défaut
        return True 


"""
Module de détection des vulnérabilités et analyse de sécurité
"""

import logging
import re
import json
import time
import os
from urllib.parse import urlparse, urljoin, parse_qs
import requests
from typing import Dict, List, Any, Optional, Set, Tuple
from bs4 import BeautifulSoup
import concurrent.futures

from ..config import CrawlerConfig

logger = logging.getLogger(__name__)

class VulnerabilityScanner:
    """
    Scanner de vulnérabilités et d'exposés sensibles
    """
    
    # Listes de chemins sensibles à vérifier
    SENSITIVE_FILES = [
        "/.git/HEAD",
        "/.git/config",
        "/.env",
        "/.env.local",
        "/.env.development",
        "/.env.production",
        "/wp-config.php",
        "/config.php",
        "/configuration.php",
        "/config.js",
        "/config.json",
        "/credentials.json",
        "/database.yml",
        "/settings.json",
        "/app.json",
        "/backup.sql",
        "/backup.zip",
        "/backup.tar.gz",
        "/dump.sql",
        "/phpinfo.php",
        "/info.php",
        "/test.php",
        "/admin/login",
        "/login.html",
        "/robots.txt",
        "/crossdomain.xml",
        "/sitemap.xml",
        "/.htaccess",
        "/.dockerignore",
        "/Dockerfile",
        "/docker-compose.yml",
        "/package.json",
        "/package-lock.json",
        "/composer.json",
        "/composer.lock",
        "/yarn.lock",
        "/README.md",
        "/CHANGELOG.md",
        "/LICENSE",
        "/node_modules/",
        "/vendor/",
        "/logs/",
        "/debug/",
        "/temp/",
        "/uploads/",
        "/backups/"
    ]
    
    # En-têtes HTTP à vérifier pour les problèmes de sécurité
    SECURITY_HEADERS = {
        "Content-Security-Policy": {
            "required": False,
            "description": "Empêche les attaques XSS en spécifiant les sources de contenu autorisées."
        },
        "X-Content-Type-Options": {
            "required": True,
            "description": "Empêche le navigateur d'interpréter les fichiers comme un type MIME différent."
        },
        "X-Frame-Options": {
            "required": True,
            "description": "Protège contre les attaques de clickjacking."
        },
        "X-XSS-Protection": {
            "required": True,
            "description": "Filtre de base pour les attaques XSS dans les navigateurs plus anciens."
        },
        "Strict-Transport-Security": {
            "required": True,
            "description": "Force les connexions via HTTPS plutôt que HTTP."
        },
        "Referrer-Policy": {
            "required": False,
            "description": "Contrôle la quantité d'informations envoyées dans l'en-tête Referer."
        },
        "Permissions-Policy": {
            "required": False,
            "description": "Contrôle les fonctionnalités du navigateur autorisées (remplace Feature-Policy)."
        },
        "Access-Control-Allow-Origin": {
            "required": False,
            "description": "Indique si la ressource peut être partagée avec une autre origine."
        }
    }
    
    def __init__(self, config: Optional[Any] = None):
        """
        Initialise le scanner de vulnérabilités.
        
        Args:
            config: Configuration du scanner
        """
        self.config = config or CrawlerConfig()
        self.session = requests.Session()
        
        # Configuration de la session
        if self.config.user_agent_rotate:
            self.session.headers.update({"User-Agent": self.config.get_user_agent()})
        else:
            self.session.headers.update({"User-Agent": self.config.user_agent or "HakBoardCrawler/1.0"})
        
        # Configuration du proxy si nécessaire
        if self.config.proxy:
            self.session.proxies = {
                "http": self.config.proxy,
                "https": self.config.proxy
            }
        
        # Ensemble pour stocker les URLs déjà visitées
        self.visited_urls = set()
        
        # Ensemble pour stocker les endpoints d'API détectés
        self.api_endpoints = set()
        
        # Dictionnaire pour le mapping du site
        self.site_map = {}
        
        logger.info("Scanner de vulnérabilités initialisé")
    
    def scan_vulnerabilities(self, target: str) -> Dict[str, Any]:
        """
        Effectue un scan de vulnérabilités sur la cible
        
        Args:
            target: URL ou domaine cible
            
        Returns:
            Dictionnaire des vulnérabilités trouvées
        """
        logger.info(f"Démarrage du scan de vulnérabilités pour {target}")
        
        # S'assurer que l'URL est complète
        if not target.startswith(("http://", "https://")):
            target = f"http://{target}"
        
        results = {
            "target": target,
            "cors_issues": [],
            "csp_issues": [],
            "header_issues": [],
            "exposed_files": [],
            "potential_vulnerabilities": []
        }
        
        # Analyser les en-têtes de sécurité
        headers_analysis = self.analyze_headers(target)
        results["header_issues"] = headers_analysis.get("issues", [])
        
        # Scanner les fichiers sensibles
        sensitive_files = self.scan_sensitive_files(target)
        results["exposed_files"] = sensitive_files.get("exposed_files", [])
        
        # Rechercher d'autres vulnérabilités potentielles
        self._scan_for_vulnerabilities(target, results)
        
        logger.info(f"Scan de vulnérabilités terminé pour {target}")
        return results
    
    def analyze_headers(self, url: str) -> Dict[str, Any]:
        """
        Analyse les en-têtes HTTP pour les problèmes de sécurité
        
        Args:
            url: URL à analyser
            
        Returns:
            Dictionnaire des problèmes d'en-têtes trouvés
        """
        logger.info(f"Analyse des en-têtes de sécurité pour {url}")
        
        results = {
            "url": url,
            "headers": {},
            "issues": [],
            "score": 0,
            "max_score": len([h for h in self.SECURITY_HEADERS.values() if h["required"]])
        }
        
        try:
            # Effectuer une requête HEAD pour récupérer les en-têtes
            response = self.session.head(url, timeout=self.config.timeout)
            
            # Analyser chaque en-tête de sécurité
            for header, details in self.SECURITY_HEADERS.items():
                header_value = response.headers.get(header)
                results["headers"][header] = header_value
                
                if details["required"] and not header_value:
                    results["issues"].append({
                        "severity": "high" if details["required"] else "medium",
                        "header": header,
                        "description": f"En-tête {header} manquant. {details['description']}"
                    })
                elif header_value:
                    # Incrémenter le score de sécurité si l'en-tête est présent
                    if details["required"]:
                        results["score"] += 1
                    
                    # Analyse spécifique pour certains en-têtes
                    if header == "Content-Security-Policy":
                        csp_issues = self._analyze_csp(header_value)
                        for issue in csp_issues:
                            results["issues"].append({
                                "severity": "medium",
                                "header": header,
                                "description": issue
                            })
                    
                    elif header == "Access-Control-Allow-Origin":
                        if header_value == "*":
                            results["issues"].append({
                                "severity": "medium",
                                "header": header,
                                "description": "CORS est configuré pour autoriser toutes les origines (*), ce qui peut présenter un risque de sécurité."
                            })
            
            # Rechercher des en-têtes potentiellement risqués
            for header, value in response.headers.items():
                if header.lower() in ["server", "x-powered-by", "x-aspnet-version", "x-aspnetmvc-version"]:
                    results["headers"][header] = value
                    results["issues"].append({
                        "severity": "low",
                        "header": header,
                        "description": f"L'en-tête {header} révèle des informations sur la technologie utilisée: {value}"
                    })
            
            logger.info(f"Analyse des en-têtes terminée avec un score de {results['score']}/{results['max_score']}")
        
        except Exception as e:
            logger.error(f"Erreur lors de l'analyse des en-têtes: {str(e)}")
            results["issues"].append({
                "severity": "error",
                "header": "N/A",
                "description": f"Erreur lors de l'analyse: {str(e)}"
            })
        
        return results
    
    def _analyze_csp(self, csp_value: str) -> List[str]:
        """
        Analyse la politique de sécurité du contenu (CSP)
        
        Args:
            csp_value: Valeur de l'en-tête CSP
            
        Returns:
            Liste des problèmes détectés
        """
        issues = []
        
        # Vérifier si unsafe-inline est utilisé
        if "unsafe-inline" in csp_value:
            issues.append("La directive 'unsafe-inline' est utilisée dans le CSP, ce qui peut permettre des attaques XSS.")
        
        # Vérifier si unsafe-eval est utilisé
        if "unsafe-eval" in csp_value:
            issues.append("La directive 'unsafe-eval' est utilisée dans le CSP, ce qui peut permettre l'exécution de code arbitraire.")
        
        # Vérifier si des sources génériques sont utilisées
        if "http:" in csp_value or "*" in csp_value:
            issues.append("Des sources génériques (http: ou *) sont utilisées dans le CSP, ce qui réduit son efficacité.")
        
        return issues
        
    def scan_sensitive_files(self, url: str) -> Dict[str, Any]:
        """
        Recherche des fichiers sensibles exposés
        
        Args:
            url: URL de base à scanner
            
        Returns:
            Dictionnaire des fichiers sensibles trouvés
        """
        logger.info(f"Recherche de fichiers sensibles pour {url}")
        
        results = {
            "url": url,
            "exposed_files": [],
            "total_checked": 0,
            "total_found": 0
        }
        
        # S'assurer que l'URL se termine par un slash
        if not url.endswith("/"):
            url = url + "/"
        
        # Créer une liste d'URLs à vérifier
        urls_to_check = [urljoin(url, path) for path in self.SENSITIVE_FILES]
        results["total_checked"] = len(urls_to_check)
        
        # Définir une fonction pour vérifier une URL spécifique
        def check_url(target_url):
            try:
                # Utiliser HEAD d'abord pour être moins intrusif
                response = self.session.head(target_url, timeout=self.config.timeout, allow_redirects=False)
                
                # Si le serveur ne supporte pas HEAD ou renvoie un code 405, essayer GET
                if response.status_code == 405:
                    response = self.session.get(target_url, timeout=self.config.timeout, allow_redirects=False, stream=True)
                    # Lire juste le début pour éviter de télécharger des fichiers volumineux
                    response.raw.read(1024)
                    response.close()
                
                # Vérifier le code de statut
                if 200 <= response.status_code < 300:
                    path = urlparse(target_url).path
                    return {
                        "url": target_url,
                        "path": path,
                        "status_code": response.status_code,
                        "content_type": response.headers.get("Content-Type", ""),
                        "content_length": response.headers.get("Content-Length", "")
                    }
                
                return None
                
            except Exception as e:
                logger.debug(f"Erreur lors de la vérification de {target_url}: {str(e)}")
                return None
        
        # Utiliser un ThreadPoolExecutor pour vérifier plusieurs URLs simultanément
        with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
            futures = [executor.submit(check_url, url) for url in urls_to_check]
            
            for future in concurrent.futures.as_completed(futures):
                try:
                    result = future.result()
                    if result:
                        results["exposed_files"].append(result)
                except Exception as e:
                    logger.error(f"Erreur dans le thread de vérification: {str(e)}")
        
        results["total_found"] = len(results["exposed_files"])
        logger.info(f"Scan terminé. {results['total_found']} fichiers sensibles trouvés sur {results['total_checked']} vérifiés.")
        
        return results
    
    def detect_api_endpoints(self, url: str) -> Dict[str, Any]:
        """
        Détecte les endpoints d'API potentiels
        
        Args:
            url: URL de base à scanner
            
        Returns:
            Dictionnaire des endpoints API détectés
        """
        logger.info(f"Recherche d'endpoints API pour {url}")
        
        results = {
            "url": url,
            "endpoints": [],
            "total_found": 0
        }
        
        # Réinitialiser les endpoints détectés
        self.api_endpoints = set()
        
        # Exploration du site pour trouver des endpoints API
        try:
            self._crawl_for_api_endpoints(url, depth=0, max_depth=self.config.scan_depth)
            
            # Convertir les résultats
            for endpoint in self.api_endpoints:
                results["endpoints"].append({
                    "url": endpoint,
                    "method": "GET",  # Par défaut, nous ne pouvons pas déterminer les méthodes sans tests
                    "parameters": self._extract_url_params(endpoint)
                })
            
            results["total_found"] = len(results["endpoints"])
            logger.info(f"Détection d'API terminée. {results['total_found']} endpoints trouvés.")
        
        except Exception as e:
            logger.error(f"Erreur lors de la détection d'endpoints API: {str(e)}")
        
        return results
    
    def _crawl_for_api_endpoints(self, url: str, depth: int = 0, max_depth: int = 3):
        """
        Explore récursivement le site pour trouver des endpoints API
        
        Args:
            url: URL à explorer
            depth: Profondeur actuelle
            max_depth: Profondeur maximale d'exploration
        """
        # Vérifier si nous sommes trop profonds ou si l'URL a déjà été visitée
        if depth > max_depth or url in self.visited_urls:
            return
        
        # Marquer l'URL comme visitée
        self.visited_urls.add(url)
        
        try:
            # Télécharger la page
            response = self.session.get(url, timeout=self.config.timeout)
            if response.status_code != 200:
                return
            
            # Analyser le contenu
            soup = BeautifulSoup(response.text, "html.parser")
            
            # Rechercher des motifs d'API dans les scripts
            scripts = soup.find_all("script")
            for script in scripts:
                if script.string:
                    # Rechercher des patterns d'URL d'API
                    api_patterns = [
                        r'["\'](https?:\/\/[^"\']*api[^"\']*)["\']',
                        r'["\'](\/api\/[^"\']*)["\']',
                        r'["\'](\/v\d+\/[^"\']*)["\']',
                        r'["\'](\/rest\/[^"\']*)["\']',
                        r'["\'](\/graphql[^"\']*)["\']',
                        r'["\'](\/gql[^"\']*)["\']'
                    ]
                    
                    # Patterns spécifiques aux frameworks
                    framework_patterns = [
                        # Next.js
                        r'["\'](\/\_next\/data\/[^"\']*)["\']',
                        r'["\'](\/\_next\/static\/[^"\']*)["\']',
                        r'["\'](\/pages\/api\/[^"\']*)["\']',
                        # Django
                        r'["\'](\/admin\/api\/[^"\']*)["\']',
                        r'["\'](\/django-admin\/[^"\']*)["\']',
                        # WordPress
                        r'["\'](\/wp-json\/[^"\']*)["\']',
                        r'["\'](\/wp-admin\/admin-ajax.php)["\']',
                        # Vue/Vite
                        r'["\'](\/src\/api\/[^"\']*)["\']',
                        r'["\'](\/assets\/[^"\']*\.js)["\']',
                        # Laravel
                        r'["\'](\/api\/[^"\']*)["\']',
                        r'["\'](\/sanctum\/csrf-cookie)["\']',
                        # React
                        r'["\'](\/static\/js\/[^"\']*)["\']',
                        # Angular
                        r'["\'](\/assets\/[^"\']*)["\']',
                        r'["\'](\/api\/[^"\']*)["\']'
                    ]
                    
                    # Ajouter les patterns de frameworks
                    api_patterns.extend(framework_patterns)
                    
                    for pattern in api_patterns:
                        for match in re.finditer(pattern, script.string):
                            api_url = match.group(1)
                            if not api_url.startswith(("http://", "https://")):
                                parsed_url = urlparse(url)
                                base_url = f"{parsed_url.scheme}://{parsed_url.netloc}"
                                api_url = urljoin(base_url, api_url)
                            
                            self.api_endpoints.add(api_url)
            
            # Détecter les frameworks courants
            self._detect_frameworks(soup, url)
            
            # Collecter les liens pour exploration
            links = []
            for a in soup.find_all("a", href=True):
                href = a["href"]
                if not href.startswith(("http://", "https://")):
                    parsed_url = urlparse(url)
                    base_url = f"{parsed_url.scheme}://{parsed_url.netloc}"
                    href = urljoin(base_url, href)
                
                # Vérifier si le lien est sur le même domaine
                if urlparse(url).netloc == urlparse(href).netloc:
                    links.append(href)
            
            # Explorer les liens trouvés
            for link in links[:10]:  # Limiter à 10 liens par page pour éviter l'explosion
                self._crawl_for_api_endpoints(link, depth + 1, max_depth)
        
        except Exception as e:
            logger.error(f"Erreur lors de l'exploration pour API endpoints: {str(e)}")
    
    def _detect_frameworks(self, soup: BeautifulSoup, url: str):
        """
        Détecte les frameworks utilisés et explore les chemins spécifiques
        
        Args:
            soup: Objet BeautifulSoup de la page
            url: URL de base
        """
        parsed_url = urlparse(url)
        base_url = f"{parsed_url.scheme}://{parsed_url.netloc}"
        
        # Détection de Next.js
        next_indicators = [
            soup.find("div", id="__next"),
            soup.find("script", src=lambda s: s and "_next/static" in s),
            soup.find("link", href=lambda h: h and "_next/static" in h)
        ]
        if any(next_indicators):
            logger.info(f"Framework Next.js détecté sur {url}")
            # Explorer les chemins spécifiques à Next.js
            next_paths = ["/_next/data/", "/_next/static/", "/api/"]
            for path in next_paths:
                api_url = urljoin(base_url, path)
                self.api_endpoints.add(api_url)
        
        # Détection de WordPress
        wp_indicators = [
            soup.find("link", attrs={"rel": "https://api.w.org/"}),
            soup.find("meta", attrs={"name": "generator", "content": lambda c: c and "WordPress" in c}),
            soup.find("script", src=lambda s: s and "wp-includes" in s)
        ]
        if any(wp_indicators):
            logger.info(f"Framework WordPress détecté sur {url}")
            # Explorer les chemins spécifiques à WordPress
            wp_paths = ["/wp-json/wp/v2/", "/wp-admin/admin-ajax.php"]
            for path in wp_paths:
                api_url = urljoin(base_url, path)
                self.api_endpoints.add(api_url)
        
        # Détection de Django
        django_indicators = [
            soup.find("meta", attrs={"name": "generator", "content": lambda c: c and "Django" in c}),
            soup.find("script", src=lambda s: s and "django" in s)
        ]
        if any(django_indicators):
            logger.info(f"Framework Django détecté sur {url}")
            # Explorer les chemins spécifiques à Django
            django_paths = ["/admin/", "/api/", "/static/"]
            for path in django_paths:
                api_url = urljoin(base_url, path)
                self.api_endpoints.add(api_url)
        
        # Détection de Laravel
        laravel_indicators = [
            soup.find("meta", attrs={"name": "csrf-token"}),
            soup.find("script", string=lambda s: s and "Laravel" in s),
            soup.find("script", src=lambda s: s and "vendor/laravel" in s)
        ]
        if any(laravel_indicators):
            logger.info(f"Framework Laravel détecté sur {url}")
            # Explorer les chemins spécifiques à Laravel
            laravel_paths = ["/api/", "/sanctum/csrf-cookie", "/storage/"]
            for path in laravel_paths:
                api_url = urljoin(base_url, path)
                self.api_endpoints.add(api_url)
        
        # Détection de Vue/Vite
        vue_indicators = [
            soup.find("div", id="app"),
            soup.find("script", src=lambda s: s and ".vue" in s),
            soup.find("script", src=lambda s: s and "vue.js" in s),
            soup.find("script", src=lambda s: s and "vite" in s)
        ]
        if any(vue_indicators):
            logger.info(f"Framework Vue.js/Vite détecté sur {url}")
            # Explorer les chemins spécifiques à Vue.js/Vite
            vue_paths = ["/src/api/", "/assets/"]
            for path in vue_paths:
                api_url = urljoin(base_url, path)
                self.api_endpoints.add(api_url)
    
    def _extract_url_params(self, url: str) -> List[str]:
        """
        Extrait les paramètres d'une URL
        
        Args:
            url: URL à analyser
            
        Returns:
            Liste des paramètres détectés
        """
        parsed_url = urlparse(url)
        params = []
        
        if parsed_url.query:
            query_params = parse_qs(parsed_url.query)
            for param in query_params.keys():
                params.append(param)
        
        return params
    
    def _scan_for_vulnerabilities(self, url: str, results: Dict[str, Any]):
        """
        Recherche d'autres vulnérabilités potentielles
        
        Args:
            url: URL à scanner
            results: Dictionnaire pour stocker les résultats
        """
        # Ce code serait plus complexe dans une implémentation réelle
        # Ici, nous simulons la détection de quelques vulnérabilités courantes
        
        try:
            # Simuler la détection de certaines vulnérabilités
            # Dans une implémentation réelle, il y aurait des tests plus poussés
            
            response = self.session.get(url, timeout=self.config.timeout)
            
            # Vérifier les formulaires sans CSRF protection
            soup = BeautifulSoup(response.text, "html.parser")
            forms = soup.find_all("form")
            
            for form in forms:
                # Vérifier si un token CSRF est présent
                csrf_fields = form.find_all("input", attrs={"name": re.compile("csrf|token", re.I)})
                if not csrf_fields:
                    results["potential_vulnerabilities"].append({
                        "type": "csrf",
                        "severity": "medium",
                        "url": url,
                        "description": "Formulaire potentiellement vulnérable aux attaques CSRF"
                    })
            
            # Vérifier les cookies sans attributs de sécurité
            if "Set-Cookie" in response.headers:
                cookies = response.headers.get("Set-Cookie")
                if "secure" not in cookies.lower() or "httponly" not in cookies.lower():
                    results["potential_vulnerabilities"].append({
                        "type": "insecure_cookie",
                        "severity": "medium",
                        "url": url,
                        "description": "Cookies sans attributs de sécurité (Secure, HttpOnly)"
                    })
            
            logger.info(f"Scan de vulnérabilités supplémentaires terminé pour {url}")
            
        except Exception as e:
            logger.error(f"Erreur lors du scan de vulnérabilités: {str(e)}")
    
    def generate_site_map(self, url: str) -> Dict[str, Any]:
        """
        Génère une carte du site
        
        Args:
            url: URL de base à explorer
            
        Returns:
            Dictionnaire contenant la carte du site
        """
        logger.info(f"Génération de la carte du site pour {url}")
        
        site_map = {
            "url": url,
            "nodes": [],
            "resources": {
                "images": [],
                "scripts": [],
                "stylesheets": [],
                "links": [],
                "forms": [],
                "frames": [],
                "fonts": [],
                "media": []
            },
            "total_pages": 0,
            "structure": {
                "name": "Root",
                "children": {}
            }
        }
        
        visited = set()
        queue = [(url, 0)]  # (url, depth)
        
        max_depth = self.config.scan_depth
        max_pages = 100  # Limiter le nombre de pages pour éviter une exploration trop longue
        
        # Structure pour organiser l'arborescence
        def add_to_structure(url_path: str, page_url: str, title: str = ""):
            parts = url_path.strip('/').split('/')
            current = site_map["structure"]
            
            # Ajouter chaque partie du chemin dans l'arborescence
            for i, part in enumerate(parts):
                if not part:  # Ignorer les parties vides
                    continue
                    
                # Si cette partie n'existe pas encore, la créer
                if part not in current["children"]:
                    current["children"][part] = {
                        "name": part,
                        "children": {},
                        "urls": []
                    }
                
                current = current["children"][part]
            
            # Ajouter l'URL au nœud actuel
            if "urls" not in current:
                current["urls"] = []
                
            current["urls"].append({
                "url": page_url,
                "title": title or page_url
            })
        
        while queue and len(visited) < max_pages:
            current_url, depth = queue.pop(0)
            
            if current_url in visited or depth > max_depth:
                continue
                
            visited.add(current_url)
            
            try:
                response = self.session.get(
                    current_url, 
                    timeout=self.config.timeout,
                    allow_redirects=self.config.follow_redirects
                )
                
                if response.status_code != 200:
                    continue
                
                # Vérifier si c'est une page HTML
                content_type = response.headers.get("Content-Type", "").lower()
                if "text/html" not in content_type:
                    continue
                
                # Parser le HTML
                soup = BeautifulSoup(response.text, "html.parser")
                
                # Extraire le titre
                title = soup.find("title")
                title_text = title.text.strip() if title else ""
                
                # Ajouter à la structure
                parsed_url = urlparse(current_url)
                path = parsed_url.path or "/"
                add_to_structure(path, current_url, title_text)
                
                # Ajouter aux nœuds
                node = {
                    "url": current_url,
                    "title": title_text,
                    "depth": depth,
                    "status": response.status_code,
                    "content_type": content_type,
                    "links": []
                }
                
                # Extraire les liens
                for a in soup.find_all("a", href=True):
                    href = a["href"]
                    
                    # Ignorer les ancres et les liens JavaScript
                    if href.startswith("#") or href.startswith("javascript:"):
                        continue
                    
                    # Résoudre les URLs relatives
                    if not href.startswith(("http://", "https://")):
                        href = urljoin(current_url, href)
                    
                    # Filtrer pour ne garder que les liens du même domaine
                    href_parsed = urlparse(href)
                    current_parsed = urlparse(current_url)
                    
                    is_external = href_parsed.netloc != current_parsed.netloc
                    
                    link = {
                        "url": href,
                        "text": a.text.strip(),
                        "is_external": is_external
                    }
                    
                    node["links"].append(link)
                    site_map["resources"]["links"].append(link)
                    
                    # Ajouter à la file d'attente si c'est un lien interne
                    if not is_external and href not in visited:
                        queue.append((href, depth + 1))
                
                # Extraire les images (si activé)
                if self.config.sitemap_options.get("extract_images", True):
                    for img in soup.find_all("img", src=True):
                        src = img["src"]
                        if not src.startswith(("http://", "https://")):
                            src = urljoin(current_url, src)
                        
                        site_map["resources"]["images"].append({
                            "src": src,
                            "alt": img.get("alt", ""),
                            "page": current_url
                        })
                
                # Extraire les scripts (si activé)
                if self.config.sitemap_options.get("extract_scripts", True):
                    for script in soup.find_all("script", src=True):
                        src = script["src"]
                        if not src.startswith(("http://", "https://")):
                            src = urljoin(current_url, src)
                        
                        site_map["resources"]["scripts"].append({
                            "src": src,
                            "page": current_url
                        })
                
                # Extraire les feuilles de style (si activé)
                if self.config.sitemap_options.get("extract_styles", True):
                    for link in soup.find_all("link", rel="stylesheet", href=True):
                        href = link["href"]
                        if not href.startswith(("http://", "https://")):
                            href = urljoin(current_url, href)
                        
                        site_map["resources"]["stylesheets"].append({
                            "href": href,
                            "page": current_url
                        })
                
                # Extraire les formulaires (si activé)
                if self.config.sitemap_options.get("extract_forms", True):
                    for form in soup.find_all("form"):
                        action = form.get("action", "")
                        if action and not action.startswith(("http://", "https://")):
                            action = urljoin(current_url, action)
                        
                        site_map["resources"]["forms"].append({
                            "action": action,
                            "method": form.get("method", "get"),
                            "page": current_url,
                            "fields": len(form.find_all(["input", "select", "textarea"]))
                        })
                
                # Extraire les iframes (si activé)
                if self.config.sitemap_options.get("extract_frames", True):
                    for iframe in soup.find_all("iframe", src=True):
                        src = iframe["src"]
                        if not src.startswith(("http://", "https://")):
                            src = urljoin(current_url, src)
                        
                        site_map["resources"]["frames"].append({
                            "src": src,
                            "page": current_url
                        })
                
                # Ajouter le nœud à la carte du site
                site_map["nodes"].append(node)
                
                # Respecter le délai entre les requêtes
                time.sleep(self.config.delay)
                
            except Exception as e:
                logger.error(f"Erreur lors de l'exploration de {current_url}: {str(e)}")
        
        # Mettre à jour le nombre total de pages
        site_map["total_pages"] = len(site_map["nodes"])
        
        # Ajouter des statistiques
        resources = site_map["resources"]
        site_map["stats"] = {
            "total_pages": len(site_map["nodes"]),
            "total_links": len(resources["links"]),
            "total_images": len(resources["images"]),
            "total_scripts": len(resources["scripts"]),
            "total_styles": len(resources["stylesheets"]),
            "total_forms": len(resources["forms"]),
            "total_frames": len(resources["frames"]),
            "max_depth": max(node["depth"] for node in site_map["nodes"]) if site_map["nodes"] else 0
        }
        
        logger.info(f"Carte du site générée avec {site_map['total_pages']} pages")
        return site_map 


"""
HakBoardCrawler - Un scanner de vulnérabilités web avancé pour le Red et Blue Teaming
"""

__version__ = '1.0.0'
__author__ = 'HakBoard Team'

from .crawler import Crawler
from .config import CrawlerConfig

__all__ = ["Crawler", "CrawlerConfig", "__version__"] 


"""
Module pour simuler un comportement humain et éviter la détection
"""

import logging
import time
import random
from typing import Dict, Any, List, Optional

logger = logging.getLogger(__name__)

class StealthMode:
    """
    Techniques pour simuler un comportement humain et éviter la détection
    """
    
    def __init__(self, config):
        """
        Initialise le mode furtif
        
        Args:
            config: Configuration du crawler
        """
        self.config = config
        logger.info("Mode furtif initialisé")
    
    def apply_stealth_techniques(self):
        """
        Applique les techniques de furtivité selon la configuration
        """
        if self.config.mode == "stealth":
            logger.info("Application des techniques de furtivité")
            
            # Appliquer les différentes techniques configurées
            if self.config.get("stealth_features.random_timing"):
                self._apply_random_timing()
            
            if self.config.get("stealth_features.mouse_movement"):
                self._simulate_mouse_movement()
            
            if self.config.get("stealth_features.anti_fingerprinting"):
                self._apply_anti_fingerprinting()
            
            logger.info("Techniques de furtivité appliquées")
        else:
            logger.info("Mode furtif désactivé (mode agressif)")
    
    def _apply_random_timing(self):
        """
        Applique des délais aléatoires entre les requêtes pour simuler un comportement humain
        """
        logger.debug("Application de délais aléatoires")
        
        # Calculer un délai aléatoire basé sur le délai configuré
        base_delay = self.config.request_delay
        jitter = base_delay * 0.5  # 50% de variation
        random_delay = max(0.5, random.uniform(base_delay - jitter, base_delay + jitter))
        
        logger.debug(f"Pause de {random_delay:.2f} secondes pour simuler un comportement humain")
        time.sleep(random_delay)
    
    def _simulate_mouse_movement(self):
        """
        Simule des mouvements de souris pour les sites qui détectent l'automatisation
        """
        logger.debug("Simulation de mouvements de souris")
        
        # Cette fonctionnalité nécessite PyAutoGUI en environnement graphique
        # Dans un environnement sans interface graphique, cela est simplement ignoré
        try:
            import pyautogui
            
            # Obtenir la taille de l'écran
            screen_width, screen_height = pyautogui.size()
            
            # Générer un chemin aléatoire de points
            num_points = random.randint(5, 10)
            points = [(random.randint(0, screen_width), random.randint(0, screen_height)) 
                     for _ in range(num_points)]
            
            # Déplacer la souris le long du chemin
            current_x, current_y = pyautogui.position()
            for x, y in points:
                # Calculer la distance pour que le mouvement soit plus naturel
                distance = ((x - current_x) ** 2 + (y - current_y) ** 2) ** 0.5
                duration = distance / 1000.0  # Plus la distance est grande, plus le mouvement est lent
                
                # Limite pour éviter les mouvements trop lents
                duration = min(duration, 1.0)
                
                # Déplacer la souris avec une courbe naturelle
                pyautogui.moveTo(x, y, duration=duration, tween=pyautogui.easeInOutQuad)
                
                # Mettre à jour la position actuelle
                current_x, current_y = x, y
                
                # Petite pause aléatoire
                time.sleep(random.uniform(0.1, 0.3))
            
            logger.debug("Mouvements de souris simulés avec succès")
        except ImportError:
            logger.warning("PyAutoGUI non disponible, simulation de souris ignorée")
        except Exception as e:
            logger.error(f"Erreur lors de la simulation de mouvements de souris: {str(e)}")
    
    def _apply_anti_fingerprinting(self):
        """
        Applique des techniques pour éviter la détection par fingerprinting
        """
        logger.debug("Application des techniques anti-fingerprinting")
        
        # Ces techniques sont généralement intégrées dans des outils comme Selenium ou Playwright
        # Ici, nous simulons simplement leur application dans cet exemple
        
        # Techniques courantes:
        # 1. Rotation d'User-Agent (déjà géré dans la classe principale)
        # 2. Navigation via proxy (déjà géré dans la classe principale)
        # 3. Modifications des propriétés navigator en JavaScript (nécessite un navigateur)
        # 4. Modification des Canvas fingerprints (nécessite un navigateur)
        # 5. Modification des propriétés WebGL (nécessite un navigateur)
        
        logger.info("Techniques anti-fingerprinting appliquées (simulation)")
    
    def get_random_user_agent(self) -> str:
        """
        Génère un User-Agent aléatoire
        
        Returns:
            User-Agent aléatoire
        """
        try:
            from fake_useragent import UserAgent
            ua = UserAgent()
            
            # Sélectionner un type de navigateur selon la configuration
            browser = self.config.get("emulate_browser", "random")
            
            if browser == "chrome":
                return ua.chrome
            elif browser == "firefox":
                return ua.firefox
            elif browser == "safari":
                return ua.safari
            elif browser == "edge":
                return ua.edge
            else:
                return ua.random
            
        except ImportError:
            logger.warning("Module fake_useragent non disponible")
            
            # User-Agents par défaut
            user_agents = [
                # Chrome
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
                "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.114 Safari/537.36",
                # Firefox
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:89.0) Gecko/20100101 Firefox/89.0",
                "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:89.0) Gecko/20100101 Firefox/89.0",
                # Safari
                "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1.1 Safari/605.1.15",
                # Edge
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36 Edg/91.0.864.59"
            ]
            
            return random.choice(user_agents) 


"""
Interface en ligne de commande pour HakBoardCrawler
"""

import argparse
import logging
import sys
import json
from typing import List, Optional

from . import Crawler, __version__

def setup_logging(verbose: bool = False):
    """
    Configure le système de logging
    """
    level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(
        level=level,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )

def parse_args(args: Optional[List[str]] = None):
    """
    Parse les arguments de la ligne de commande
    """
    parser = argparse.ArgumentParser(
        description="HakBoardCrawler - Un crawler avancé pour le Red/Blue Teaming"
    )
    parser.add_argument(
        "--version", action="version", version=f"HakBoardCrawler v{__version__}"
    )
    parser.add_argument(
        "-v", "--verbose", action="store_true", help="Mode verbeux"
    )
    
    subparsers = parser.add_subparsers(dest="command", help="Commandes disponibles")
    
    # Commande 'scan'
    scan_parser = subparsers.add_parser("scan", help="Scanner une cible")
    scan_parser.add_argument("target", help="URL ou domaine cible")
    scan_parser.add_argument(
        "--mode", choices=["stealth", "aggressive"], default="stealth",
        help="Mode de scan (défaut: stealth)"
    )
    scan_parser.add_argument(
        "--metadata", action="store_true", 
        help="Activer l'extraction de métadonnées"
    )
    scan_parser.add_argument(
        "--social", action="store_true", 
        help="Activer le scraping des réseaux sociaux"
    )
    scan_parser.add_argument(
        "--depth", type=int, default=3,
        help="Profondeur de scan (défaut: 3)"
    )
    scan_parser.add_argument(
        "--timeout", type=int, default=30,
        help="Timeout des requêtes en secondes (défaut: 30)"
    )
    scan_parser.add_argument(
        "--delay", type=float,
        help="Délai entre les requêtes en secondes"
    )
    scan_parser.add_argument(
        "--no-robots", action="store_true",
        help="Ne pas respecter robots.txt"
    )
    scan_parser.add_argument(
        "--captcha", choices=["auto", "2captcha", "disabled"], default="auto",
        help="Méthode de bypass CAPTCHA (défaut: auto)"
    )
    scan_parser.add_argument(
        "--export", default="json",
        help="Format(s) d'export séparés par virgule (json,csv,html,pdf)"
    )
    scan_parser.add_argument(
        "--output", default="rapport",
        help="Nom de base pour les fichiers exportés"
    )
    scan_parser.add_argument(
        "--proxy",
        help="URL du proxy à utiliser"
    )
    
    # Commande 'config'
    config_parser = subparsers.add_parser("config", help="Gérer la configuration")
    config_parser.add_argument(
        "--show", action="store_true",
        help="Afficher la configuration actuelle"
    )
    config_parser.add_argument(
        "--set",
        help="Définir une clé API (format: service:clé)"
    )
    
    return parser.parse_args(args)

def main(args: Optional[List[str]] = None):
    """
    Point d'entrée principal pour l'interface CLI
    """
    parsed_args = parse_args(args)
    setup_logging(parsed_args.verbose)
    
    if parsed_args.command == "scan":
        # Configuration du crawler
        kwargs = {
            "mode": parsed_args.mode,
            "scan_depth": parsed_args.depth,
            "timeout": parsed_args.timeout,
            "enable_metadata": parsed_args.metadata,
            "enable_social": parsed_args.social,
            "captcha_bypass": parsed_args.captcha,
        }
        
        if parsed_args.delay:
            kwargs["request_delay"] = parsed_args.delay
        
        if parsed_args.no_robots:
            kwargs["respect_robots_txt"] = False
            
        if parsed_args.proxy:
            kwargs["proxy"] = parsed_args.proxy
        
        # Initialisation et exécution du crawler
        crawler = Crawler(parsed_args.target, **kwargs)
        results = crawler.scan()
        
        # Export des résultats
        export_formats = [f.strip() for f in parsed_args.export.split(",")]
        for format in export_formats:
            output_file = crawler.export(results, format, parsed_args.output)
            print(f"Résultats exportés vers: {output_file}")
            
    elif parsed_args.command == "config":
        if parsed_args.show:
            # Afficher la configuration par défaut
            from .config import Config
            config = Config()
            print(json.dumps(config.config, indent=2, default=str))
            
        elif parsed_args.set:
            # La gestion complète des clés API nécessiterait un fichier de configuration persistant
            try:
                service, key = parsed_args.set.split(":", 1)
                print(f"Clé API pour {service} définie")
            except ValueError:
                print("Format invalide. Utilisez --set service:clé")
                return 1
    else:
        # Si aucune commande n'est spécifiée, afficher l'aide
        parse_args(["--help"])
    
    return 0

if __name__ == "__main__":
    sys.exit(main()) 


"""
Module pour bypasser les protections (robots.txt, captchas, etc.)
"""

import logging
import urllib.robotparser
from urllib.parse import urlparse, urljoin
import time
import random
import re
import requests
import base64
import tempfile
import os
from typing import Dict, Any, List, Optional, Tuple

logger = logging.getLogger(__name__)

class RobotsBypass:
    """
    Gère le bypass de robots.txt et autres protections
    """
    
    def __init__(self, config):
        """
        Initialise le gestionnaire de bypass
        
        Args:
            config: Configuration du crawler
        """
        self.config = config
        self.session = requests.Session()
        
        # Configuration de l'user-agent
        if self.config.user_agent_rotation:
            try:
                from fake_useragent import UserAgent
                self.ua = UserAgent()
                self.session.headers.update({"User-Agent": self.ua.random})
            except ImportError:
                logger.warning("Module fake_useragent non disponible. Utilisation de l'user-agent par défaut.")
                self.session.headers.update({
                    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
                })
        else:
            self.session.headers.update({
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
            })
        
        # Configuration du proxy si nécessaire
        if self.config.proxy:
            self.session.proxies = {
                "http": self.config.proxy,
                "https": self.config.proxy
            }
        
        # Dictionnaire pour stocker le résultat de l'analyse robots.txt (cache)
        self.robots_cache = {}
        
        logger.info("Gestionnaire de bypass initialisé")
    
    def check_robots(self, url: str) -> Dict[str, Any]:
        """
        Analyse le fichier robots.txt d'un site
        
        Args:
            url: URL du site à analyser
            
        Returns:
            Dictionnaire avec le résultat de l'analyse
        """
        parsed_url = urlparse(url)
        domain = f"{parsed_url.scheme}://{parsed_url.netloc}"
        
        # Vérifier si nous avons déjà analysé ce domaine
        if domain in self.robots_cache:
            logger.debug(f"Utilisation du cache robots.txt pour {domain}")
            return self.robots_cache[domain]
        
        results = {
            "domain": domain,
            "has_robots_txt": False,
            "respects_robots": self.config.respect_robots_txt,
            "disallowed_paths": [],
            "allowed_paths": [],
            "crawl_delay": None,
            "sitemaps": []
        }
        
        # Si nous sommes configurés pour ignorer robots.txt, retourner les résultats minimaux
        if not self.config.respect_robots_txt:
            logger.info(f"Ignorer robots.txt pour {domain} (configuration)")
            self.robots_cache[domain] = results
            return results
        
        try:
            robots_url = f"{domain}/robots.txt"
            logger.info(f"Vérification de robots.txt: {robots_url}")
            
            response = self.session.get(robots_url, timeout=self.config.timeout)
            
            if response.status_code == 200:
                results["has_robots_txt"] = True
                
                # Utiliser robotparser pour analyser le fichier
                rp = urllib.robotparser.RobotFileParser()
                rp.set_url(robots_url)
                rp.parse(response.text.splitlines())
                
                # Récupérer le user-agent actuel
                current_ua = self.session.headers.get("User-Agent", "*")
                
                # Extraire le crawl-delay s'il est défini
                for line in response.text.splitlines():
                    if "crawl-delay" in line.lower() and ":" in line:
                        try:
                            delay_str = line.split(":", 1)[1].strip()
                            results["crawl_delay"] = float(delay_str)
                        except (ValueError, IndexError):
                            pass
                
                # Extraire les sitemaps
                for line in response.text.splitlines():
                    if line.lower().startswith("sitemap:"):
                        try:
                            sitemap_url = line.split(":", 1)[1].strip()
                            results["sitemaps"].append(sitemap_url)
                        except IndexError:
                            pass
                
                # Tester quelques chemins communs pour voir s'ils sont autorisés
                common_paths = [
                    "/",
                    "/about",
                    "/contact",
                    "/wp-admin/",
                    "/wp-content/",
                    "/admin/",
                    "/login",
                    "/api/",
                    "/.git/",
                    "/.env",
                    "/config",
                    "/backup"
                ]
                
                for path in common_paths:
                    full_url = urljoin(domain, path)
                    if rp.can_fetch(current_ua, full_url):
                        results["allowed_paths"].append(path)
                    else:
                        results["disallowed_paths"].append(path)
                
                logger.info(f"Robots.txt analysé pour {domain}")
            else:
                logger.info(f"Pas de robots.txt trouvé pour {domain}")
        
        except Exception as e:
            logger.error(f"Erreur lors de l'analyse de robots.txt: {str(e)}")
        
        # Mettre en cache les résultats
        self.robots_cache[domain] = results
        return results
    
    def can_access_url(self, url: str) -> bool:
        """
        Vérifie si l'URL peut être consultée selon robots.txt
        
        Args:
            url: URL à vérifier
            
        Returns:
            True si l'URL peut être consultée, False sinon
        """
        # Si nous sommes configurés pour ignorer robots.txt, toujours autoriser
        if not self.config.respect_robots_txt:
            return True
        
        parsed_url = urlparse(url)
        domain = f"{parsed_url.scheme}://{parsed_url.netloc}"
        
        # Vérifier si nous avons déjà analysé ce domaine
        if domain not in self.robots_cache:
            self.check_robots(url)
        
        # Si le site n'a pas de robots.txt, autoriser
        if not self.robots_cache[domain]["has_robots_txt"]:
            return True
        
        # Vérifier si le chemin est dans les chemins interdits
        path = parsed_url.path
        for disallowed in self.robots_cache[domain]["disallowed_paths"]:
            if path.startswith(disallowed):
                logger.debug(f"URL {url} interdite par robots.txt")
                return False
        
        return True
    
    def solve_captcha(self, url: str, html_content: str = None) -> Optional[Dict[str, Any]]:
        """
        Tente de détecter et résoudre les CAPTCHAs
        
        Args:
            url: URL de la page avec CAPTCHA
            html_content: Contenu HTML de la page (optionnel)
            
        Returns:
            Dictionnaire avec la solution du CAPTCHA ou None
        """
        # Si les CAPTCHAs sont désactivés, retourner None
        if self.config.captcha_bypass == "disabled":
            logger.info("Bypass de CAPTCHA désactivé par configuration")
            return None
        
        # Si nous n'avons pas le contenu HTML, le télécharger
        if not html_content:
            try:
                response = self.session.get(url, timeout=self.config.timeout)
                html_content = response.text
            except Exception as e:
                logger.error(f"Erreur lors du téléchargement de la page pour CAPTCHA: {str(e)}")
                return None
        
        # Détecter le type de CAPTCHA
        captcha_type = self._detect_captcha_type(html_content)
        
        if not captcha_type:
            logger.debug(f"Pas de CAPTCHA détecté sur {url}")
            return None
        
        logger.info(f"CAPTCHA de type {captcha_type} détecté sur {url}")
        
        # Essayer de résoudre selon le type
        if captcha_type == "recaptcha" and self.config.captcha_bypass == "2captcha":
            return self._solve_recaptcha_with_2captcha(url, html_content)
        elif captcha_type == "image" and self.config.captcha_bypass == "auto":
            return self._solve_image_captcha(url, html_content)
        else:
            logger.warning(f"Pas de méthode disponible pour résoudre le CAPTCHA de type {captcha_type}")
            return None
    
    def _detect_captcha_type(self, html_content: str) -> Optional[str]:
        """
        Détecte le type de CAPTCHA dans une page
        
        Args:
            html_content: Contenu HTML de la page
            
        Returns:
            Type de CAPTCHA détecté ou None
        """
        # Détecter reCAPTCHA
        if "google.com/recaptcha/api.js" in html_content:
            return "recaptcha"
        
        # Détecter hCaptcha
        if "hcaptcha.com/1/api.js" in html_content:
            return "hcaptcha"
        
        # Détecter CAPTCHA image (basique)
        if re.search(r'captcha.*?\.(?:png|jpg|jpeg|gif)', html_content, re.IGNORECASE):
            return "image"
        
        # Pas de CAPTCHA détecté
        return None
    
    def _solve_recaptcha_with_2captcha(self, url: str, html_content: str) -> Optional[Dict[str, Any]]:
        """
        Tente de résoudre un reCAPTCHA avec le service 2captcha
        
        Args:
            url: URL de la page avec reCAPTCHA
            html_content: Contenu HTML de la page
            
        Returns:
            Dictionnaire avec la solution ou None
        """
        # Vérifier si nous avons une clé API 2captcha
        api_key = self.config.get("api_keys.2captcha")
        if not api_key:
            logger.warning("Clé API 2captcha manquante pour résoudre le reCAPTCHA")
            return None
        
        try:
            # Extraire la clé du site reCAPTCHA
            site_key_match = re.search(r'data-sitekey="([^"]+)"', html_content)
            if not site_key_match:
                logger.warning("Impossible de trouver la clé du site reCAPTCHA")
                return None
            
            site_key = site_key_match.group(1)
            
            # Ici, intégration avec l'API 2captcha
            # Dans une implémentation réelle, utiliser la librairie 2captcha-python
            logger.info("Support 2captcha non implémenté complètement dans cette version de démonstration")
            
            # Simulation d'une réponse
            return {
                "type": "recaptcha",
                "solution": "03AGdBq24PBCbwiDRgC3...KlasG38",
                "success": True
            }
            
        except Exception as e:
            logger.error(f"Erreur lors de la résolution du reCAPTCHA: {str(e)}")
            return None
    
    def _solve_image_captcha(self, url: str, html_content: str) -> Optional[Dict[str, Any]]:
        """
        Tente de résoudre un CAPTCHA image simple
        
        Args:
            url: URL de la page avec CAPTCHA
            html_content: Contenu HTML de la page
            
        Returns:
            Dictionnaire avec la solution ou None
        """
        try:
            # Rechercher l'URL de l'image CAPTCHA
            captcha_img_match = re.search(r'src="([^"]+captcha[^"]+\.(png|jpg|jpeg|gif))"', html_content, re.IGNORECASE)
            if not captcha_img_match:
                logger.warning("Impossible de trouver l'image CAPTCHA")
                return None
            
            captcha_img_url = captcha_img_match.group(1)
            if not (captcha_img_url.startswith("http://") or captcha_img_url.startswith("https://")):
                parsed_url = urlparse(url)
                base_url = f"{parsed_url.scheme}://{parsed_url.netloc}"
                captcha_img_url = urljoin(base_url, captcha_img_url)
            
            # Télécharger l'image
            response = self.session.get(captcha_img_url, timeout=self.config.timeout)
            if response.status_code != 200:
                logger.warning(f"Impossible de télécharger l'image CAPTCHA: {response.status_code}")
                return None
            
            # Utiliser OCR pour résoudre l'image (nécessite pytesseract)
            try:
                import pytesseract
                from PIL import Image
                from io import BytesIO
                
                img = Image.open(BytesIO(response.content))
                
                # Prétraitement de l'image (simplifié)
                img = img.convert('L')  # Conversion en niveaux de gris
                
                # OCR
                captcha_text = pytesseract.image_to_string(img, config='--psm 8 -c tessedit_char_whitelist=0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ')
                captcha_text = captcha_text.strip()
                
                if captcha_text:
                    logger.info(f"CAPTCHA image résolu: {captcha_text}")
                    return {
                        "type": "image",
                        "solution": captcha_text,
                        "success": True
                    }
                else:
                    logger.warning("Échec de la résolution du CAPTCHA image")
                    return None
                
            except ImportError:
                logger.warning("pytesseract non disponible pour résoudre le CAPTCHA image")
                return None
                
        except Exception as e:
            logger.error(f"Erreur lors de la résolution du CAPTCHA image: {str(e)}")
            return None 


"""
Module principal du Crawler
"""

import logging
import datetime
import os
import time
from typing import Dict, Any, List, Optional
from urllib.parse import urlparse

from .config import CrawlerConfig
from .modules.scanner import VulnerabilityScanner
from .modules.metadata import MetadataExtractor
from .modules.export import Exporter

logger = logging.getLogger(__name__)

class Crawler:
    """
    Classe principale qui coordonne l'exécution des différents modules
    """
    
    def __init__(self, target: str, mode: str = "stealth", config_file: Optional[str] = None):
        """
        Initialise le crawler
        
        Args:
            target: URL ou domaine cible
            mode: Mode d'exécution ("stealth" ou "aggressive")
            config_file: Chemin vers un fichier de configuration
        """
        # S'assurer que l'URL est complète
        if not target.startswith(("http://", "https://")):
            target = f"http://{target}"
        
        self.target = target
        self.mode = mode
        self.timestamp = datetime.datetime.now().isoformat()
        
        # Initialiser la configuration
        self.config = CrawlerConfig(config_file)
        
        # Appliquer le mode
        if mode == "aggressive":
            self.config.scan_depth = 5
            self.config.threads = 15
            self.config.timeout = 5
            self.config.delay = 0.2
        elif mode == "stealth":
            self.config.scan_depth = 2
            self.config.threads = 3
            self.config.timeout = 10
            self.config.delay = 1.5
        
        # Initialiser les modules
        self.scanner = VulnerabilityScanner(self.config)
        self.metadata_extractor = MetadataExtractor(self.config)
        self.exporter = Exporter(self.config)
        
        logger.info(f"Crawler initialisé pour {target} en mode {mode}")
    
    def scan(self) -> Dict[str, Any]:
        """
        Effectue un scan complet de la cible
        
        Returns:
            Dictionnaire des résultats du scan
        """
        start_time = time.time()
        logger.info(f"Démarrage du scan pour {self.target}")
        
        results = {
            "target": self.target,
            "timestamp": datetime.datetime.now().isoformat(),
            "mode": self.mode
        }
        
        # Scanner les vulnérabilités
        vulnerabilities = self.scanner.scan_vulnerabilities(self.target)
        results["vulnerabilities"] = vulnerabilities
        
        # Extraire les problèmes d'en-têtes pour faciliter l'accès
        results["header_issues"] = vulnerabilities.get("header_issues", [])
        results["exposed_files"] = vulnerabilities.get("exposed_files", [])
        results["potential_vulnerabilities"] = vulnerabilities.get("potential_vulnerabilities", [])
        
        # Générer la carte du site
        site_map = self.scanner.generate_site_map(self.target)
        results["site_map"] = site_map
        
        # Extraire les métadonnées
        metadata = self.metadata_extractor.extract_from_url(self.target)
        results["metadata"] = metadata
        
        # Détecter les endpoints API
        api_endpoints = self.scanner.detect_api_endpoints(self.target)
        results["endpoints"] = api_endpoints
        
        # Informations sur le framework
        results["framework_info"] = metadata.get("framework_info", {})
        results["static_resources"] = metadata.get("static_resources", {})
        
        # Générer un résumé
        results["summary"] = {
            "total_urls": site_map.get("total_pages", 0),
            "total_vulnerabilities": len(results["header_issues"]) + 
                                    len(vulnerabilities.get("cors_issues", [])) + 
                                    len(vulnerabilities.get("csp_issues", [])) + 
                                    len(results["potential_vulnerabilities"]),
            "severity_counts": self._count_vulnerabilities_by_severity(results),
            "interesting_findings": self._get_interesting_findings(results),
            "total_api_endpoints": api_endpoints.get("total_found", 0),
            "resources": {
                "images": len(results["static_resources"].get("images", [])),
                "scripts": len(results["static_resources"].get("scripts", [])),
                "stylesheets": len(results["static_resources"].get("stylesheets", [])),
                "fonts": len(results["static_resources"].get("fonts", [])),
                "media": len(results["static_resources"].get("media", []))
            },
            "scan_duration": time.time() - start_time,
            "framework": results["framework_info"].get("name", "unknown")
        }
        
        logger.info(f"Scan terminé pour {self.target} en {results['summary']['scan_duration']:.2f} secondes")
        return results
    
    def export(self, data: Dict[str, Any], format: str = "json", output: str = "rapport") -> str:
        """
        Exporte les résultats dans le format spécifié
        
        Args:
            data: Données à exporter
            format: Format d'exportation
            output: Nom de base pour le fichier de sortie
            
        Returns:
            Chemin du fichier exporté
        """
        # Ajouter le domaine au nom de fichier
        domain = urlparse(self.target).netloc
        if domain:
            output = f"{output}_{domain}"
        
        # Exporter les données
        return self.exporter.export(data, format, output)
    
    def scan_target_list(self, targets: List[str], format: str = "json", output: str = "rapport_groupe") -> List[Dict[str, Any]]:
        """
        Scanne une liste de cibles
        
        Args:
            targets: Liste des URLs ou domaines cibles
            format: Format d'exportation
            output: Nom de base pour le fichier de sortie
            
        Returns:
            Liste des résultats pour chaque cible
        """
        all_results = []
        
        for i, target in enumerate(targets):
            logger.info(f"Scan de la cible {i+1}/{len(targets)}: {target}")
            
            # Réinitialiser le crawler pour cette cible
            self.target = target if target.startswith(("http://", "https://")) else f"http://{target}"
            
            # Exécuter le scan
            results = self.scan()
            all_results.append(results)
            
            # Exporter les résultats individuels
            target_output = f"{output}_{i+1}_{urlparse(self.target).netloc}"
            self.exporter.export(results, format, target_output)
        
        # Exporter un rapport de groupe
        group_results = {
            "timestamp": datetime.datetime.now().isoformat(),
            "mode": self.mode,
            "total_targets": len(targets),
            "targets": [result["target"] for result in all_results],
            "summary": {
                "total_vulnerabilities": sum(result["summary"]["total_vulnerabilities"] for result in all_results),
                "severity_counts": {
                    "high": sum(result["summary"]["severity_counts"]["high"] for result in all_results),
                    "medium": sum(result["summary"]["severity_counts"]["medium"] for result in all_results),
                    "low": sum(result["summary"]["severity_counts"]["low"] for result in all_results),
                    "info": sum(result["summary"]["severity_counts"]["info"] for result in all_results)
                },
                "frameworks_detected": [result.get("framework_info", {}).get("name", "non détecté") for result in all_results],
                "total_api_endpoints": sum(result.get("summary", {}).get("total_api_endpoints", 0) for result in all_results),
                "total_exposed_files": sum(len(result.get("exposed_files", [])) for result in all_results)
            },
            "target_summaries": [result["summary"] for result in all_results]
        }
        
        self.exporter.export(group_results, format, f"{output}_group")
        
        return all_results
    
    def _count_vulnerabilities_by_severity(self, results: Dict[str, Any]) -> Dict[str, int]:
        """
        Compte le nombre de vulnérabilités par niveau de sévérité
        
        Args:
            results: Résultats du scan
            
        Returns:
            Dictionnaire des comptages par sévérité
        """
        counts = {"high": 0, "medium": 0, "low": 0, "info": 0}
        
        # Compter les problèmes d'en-têtes
        for issue in results.get("header_issues", []):
            severity = issue.get("severity", "info").lower()
            counts[severity] = counts.get(severity, 0) + 1
        
        # Compter les problèmes CORS
        for issue in results.get("vulnerabilities", {}).get("cors_issues", []):
            severity = issue.get("severity", "info").lower()
            counts[severity] = counts.get(severity, 0) + 1
        
        # Compter les problèmes CSP
        for issue in results.get("vulnerabilities", {}).get("csp_issues", []):
            severity = issue.get("severity", "info").lower()
            counts[severity] = counts.get(severity, 0) + 1
        
        # Compter les vulnérabilités potentielles
        for vuln in results.get("potential_vulnerabilities", []):
            severity = vuln.get("severity", "info").lower()
            counts[severity] = counts.get(severity, 0) + 1
        
        return counts
    
    def _get_interesting_findings(self, results: Dict[str, Any]) -> List[Dict[str, Any]]:
        """
        Extrait les trouvailles les plus intéressantes des résultats
        
        Args:
            results: Résultats du scan
            
        Returns:
            Liste des trouvailles intéressantes
        """
        findings = []
        
        # Ajouter les problèmes d'en-têtes à sévérité élevée
        for issue in results.get("header_issues", []):
            severity = issue.get("severity", "info").lower()
            if severity in ["high", "medium"]:
                findings.append({
                    "type": "header_issue",
                    "severity": severity,
                    "description": issue.get("description", "")
                })
        
        # Ajouter les problèmes CORS à sévérité élevée
        for issue in results.get("vulnerabilities", {}).get("cors_issues", []):
            severity = issue.get("severity", "info").lower()
            if severity in ["high", "medium"]:
                findings.append({
                    "type": "cors_issue",
                    "severity": severity,
                    "description": issue.get("description", "")
                })
        
        # Ajouter les problèmes CSP à sévérité élevée
        for issue in results.get("vulnerabilities", {}).get("csp_issues", []):
            severity = issue.get("severity", "info").lower()
            if severity in ["high", "medium"]:
                findings.append({
                    "type": "csp_issue",
                    "severity": severity,
                    "description": issue.get("description", "")
                })
        
        # Ajouter les vulnérabilités potentielles à sévérité élevée
        for vuln in results.get("potential_vulnerabilities", []):
            severity = vuln.get("severity", "info").lower()
            if severity in ["high", "medium"]:
                findings.append({
                    "type": vuln.get("type", "vulnerability"),
                    "severity": severity,
                    "description": vuln.get("description", "")
                })
        
        # Ajouter les métadonnées intéressantes
        for item in results.get("metadata", {}).get("interesting_metadata", []):
            findings.append({
                "type": "metadata",
                "file_type": item.get("type", ""),
                "filename": item.get("filename", ""),
                "interesting_fields": item.get("interesting_fields", {})
            })
        
        # Ajouter les fichiers sensibles exposés
        for file in results.get("exposed_files", []):
            findings.append({
                "type": "exposed_file",
                "severity": "high",
                "description": f"Fichier sensible exposé: {file.get('path', '')}"
            })
        
        return findings 


#!/usr/bin/env python3
"""
Script principal pour le HakBoardCrawler
"""

import logging
import argparse
import sys
import os
import time
from pathlib import Path
from typing import List, Optional

from .crawler import Crawler

def setup_logging(verbosity: int) -> None:
    """
    Configure le niveau de journalisation
    
    Args:
        verbosity: Niveau de verbosité (0-3)
    """
    log_levels = {
        0: logging.ERROR,
        1: logging.WARNING,
        2: logging.INFO,
        3: logging.DEBUG
    }
    
    level = log_levels.get(verbosity, logging.INFO)
    
    logging.basicConfig(
        level=level,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )

def validate_targets(targets: List[str]) -> List[str]:
    """
    Valide et normalise les cibles
    
    Args:
        targets: Liste des URLs ou chemin vers un fichier
        
    Returns:
        Liste des URLs normalisées
    """
    # Si c'est un fichier, le lire
    if len(targets) == 1 and os.path.isfile(targets[0]):
        try:
            with open(targets[0], 'r') as f:
                targets = [line.strip() for line in f if line.strip() and not line.startswith('#')]
        except Exception as e:
            logging.error(f"Erreur lors de la lecture du fichier de cibles: {str(e)}")
            sys.exit(1)
    
    # Normaliser les URLs
    normalized_targets = []
    for target in targets:
        # Enlever les espaces
        target = target.strip()
        
        # Vérifier si c'est une URL valide
        if not target:
            continue
            
        normalized_targets.append(target)
    
    return normalized_targets

def main() -> None:
    """
    Point d'entrée principal
    """
    parser = argparse.ArgumentParser(description='HakBoardCrawler - Un scanner de vulnérabilités web')
    
    # Arguments obligatoires
    parser.add_argument('targets', metavar='TARGET', nargs='+',
                        help='URL(s) ou domaine(s) cible(s), ou chemin vers un fichier contenant une liste de cibles')
    
    # Arguments facultatifs
    parser.add_argument('-m', '--mode', choices=['stealth', 'aggressive'], default='stealth',
                        help='Mode d\'exécution (stealth ou aggressive). Par défaut: stealth')
    parser.add_argument('-f', '--format', choices=['json', 'html', 'csv', 'pdf'], default='json',
                        help='Format d\'exportation. Par défaut: json')
    parser.add_argument('-o', '--output', default='rapport',
                        help='Préfixe du fichier de sortie. Par défaut: rapport')
    parser.add_argument('-c', '--config', help='Chemin vers le fichier de configuration')
    parser.add_argument('-d', '--depth', type=int, help='Profondeur maximale d\'exploration')
    parser.add_argument('-t', '--threads', type=int, help='Nombre de threads pour les requêtes parallèles')
    parser.add_argument('--delay', type=float, help='Délai entre les requêtes (en secondes)')
    parser.add_argument('--timeout', type=int, help='Délai d\'expiration des requêtes (en secondes)')
    parser.add_argument('--cookies', help='Cookies à inclure dans les requêtes (format: nom1=valeur1;nom2=valeur2)')
    parser.add_argument('--headers', help='En-têtes HTTP à inclure (format: nom1:valeur1;nom2:valeur2)')
    parser.add_argument('--user-agent', help='User-Agent à utiliser')
    parser.add_argument('--proxy', help='Proxy à utiliser (format: protocol://host:port)')
    parser.add_argument('-v', '--verbose', action='count', default=0, help='Augmente la verbosité')
    parser.add_argument('--no-metadata', action='store_true', help='Désactive l\'extraction de métadonnées')
    parser.add_argument('--no-sitemap', action='store_true', help='Désactive la génération de la carte du site')
    parser.add_argument('--no-api', action='store_true', help='Désactive la détection des API')
    parser.add_argument('--open', action='store_true', help='Ouvre le rapport dans le navigateur après le scan')
    
    args = parser.parse_args()
    
    # Configurer le logging
    setup_logging(args.verbose)
    
    # Valider et normaliser les cibles
    targets = validate_targets(args.targets)
    
    if not targets:
        logging.error("Aucune cible valide spécifiée")
        sys.exit(1)
    
    start_time = time.time()
    
    # Créer et configurer le crawler
    crawler = Crawler(targets[0], args.mode, args.config)
    
    # Appliquer les configurations supplémentaires
    if args.depth is not None:
        crawler.config.scan_depth = args.depth
    if args.threads is not None:
        crawler.config.threads = args.threads
    if args.delay is not None:
        crawler.config.delay = args.delay
    if args.timeout is not None:
        crawler.config.timeout = args.timeout
    if args.cookies:
        crawler.config.cookies = args.cookies
    if args.headers:
        crawler.config.headers = {
            k: v for k, v in [h.split(':', 1) for h in args.headers.split(';')]
        }
    if args.user_agent:
        crawler.config.user_agent = args.user_agent
    if args.proxy:
        crawler.config.proxy = args.proxy
    
    # Désactivation de certains modules
    if args.no_metadata:
        crawler.config.enable_metadata = False
    if args.no_sitemap:
        crawler.config.enable_sitemap = False
    if args.no_api:
        crawler.config.enable_api_detection = False
    
    try:
        # Scan unique ou multiple
        if len(targets) == 1:
            logging.info(f"Démarrage du scan de {targets[0]}")
            results = crawler.scan()
            report_path = crawler.export(results, args.format, args.output)
            logging.info(f"Scan terminé en {time.time() - start_time:.2f} secondes")
            logging.info(f"Rapport sauvegardé dans {report_path}")
            
            # Ouvrir le rapport si demandé
            if args.open and report_path:
                try:
                    import webbrowser
                    webbrowser.open(f"file://{os.path.abspath(report_path)}")
                except Exception as e:
                    logging.error(f"Erreur lors de l'ouverture du rapport: {str(e)}")
        else:
            logging.info(f"Démarrage du scan multiple pour {len(targets)} cibles")
            crawler.scan_target_list(targets, args.format, args.output)
            logging.info(f"Scan multiple terminé en {time.time() - start_time:.2f} secondes")
            logging.info(f"Rapports sauvegardés avec le préfixe {args.output}")
    
    except KeyboardInterrupt:
        logging.warning("Scan interrompu par l'utilisateur")
        sys.exit(1)
    except Exception as e:
        logging.error(f"Erreur lors du scan: {str(e)}")
        if args.verbose >= 3:
            import traceback
            traceback.print_exc()
        sys.exit(1)

if __name__ == '__main__':
    main() 


from setuptools import setup, find_packages
import os

# Lire le contenu du README.md pour la description longue
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="hakboardcrawler",
    version="1.0.0",
    description="Scanner de vulnérabilités web avancé pour le Red et Blue Teaming",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="HakBoard Team",
    author_email="contact@hakboard.com",
    packages=find_packages(),
    install_requires=[
        "requests>=2.28.0",
        "beautifulsoup4>=4.11.0",
        "fake_useragent>=1.1.0",
        "pymupdf>=1.21.0",         # Pour les PDF
        "python-docx>=0.8.11",     # Pour les documents Word
        "Pillow>=9.2.0",           # Pour les images
        "pdfkit>=1.0.0",           # Pour l'export en PDF
        "exifread>=3.0.0",         # Pour les métadonnées d'images
        "tldextract>=3.4.0",       # Pour l'analyse des domaines
        "lxml>=4.9.0",             # Pour l'analyse XML
        "jinja2>=3.1.0",           # Pour les templates HTML
        "markupsafe>=2.1.0",       # Dépendance de Jinja2
        "markdown>=3.4.0",         # Pour les conversions Markdown
        "pyyaml>=6.0.0",           # Pour le parsing de YAML
        "python-magic>=0.4.27",    # Pour la détection de type MIME
        "argparse>=1.4.0",
        "urllib3>=1.26.12",
        "certifi>=2022.9.24",
        "chardet>=5.1.0",
        "idna>=3.4",
        "cryptography>=39.0.0",    # Pour l'analyse des certificats SSL
    ],
    entry_points={
        "console_scripts": [
            "hakboardcrawler=hakboardcrawler.main:main",
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Developers",
        "Topic :: Security",
        "Topic :: Internet :: WWW/HTTP :: Indexing/Search",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.8",
) 